# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.network.v2 import floating_ip
from openstack.network.v2 import network
from openstack.network.v2 import port
from openstack.network.v2 import port_forwarding as _port_forwarding
from openstack.network.v2 import router
from openstack.network.v2 import subnet
from openstack.tests.functional import base


class TestPortForwarding(base.BaseFunctionalTest):
    IPV4 = 4
    FIP_ID: str
    EXT_CIDR = "10.100.0.0/24"
    INT_CIDR = "10.101.0.0/24"
    EXT_NET_ID: str
    INT_NET_ID: str
    EXT_SUB_ID: str
    INT_SUB_ID: str
    ROT_ID: str

    INTERNAL_PORT_ID: str
    INTERNAL_IP_ADDRESS: str
    INTERNAL_PORT = 8080
    EXTERNAL_PORT = 80
    PROTOCOL = "tcp"
    DESCRIPTION = "description"

    def setUp(self):
        super().setUp()

        if not self.user_cloud._has_neutron_extension("external-net"):
            self.skipTest(
                "Neutron external-net extension is required for this test"
            )
        if not self.user_cloud.network.find_extension(
            "floating-ip-port-forwarding"
        ):
            self.skipTest("Floating IP Port Forwarding extension disabled")

        self.ROT_NAME = self.getUniqueString()
        self.INT_NET_NAME = self.getUniqueString()
        self.INT_SUB_NAME = self.getUniqueString()

        # Find External Network
        ext_net_id = None
        for net in self.user_cloud.network.networks(is_router_external=True):
            ext_net_id = net.id
            break

        if not ext_net_id:
            self.skipTest('no external net is available')
        self.EXT_NET_ID = ext_net_id

        # Find subnet of the chosen external net
        for sub in self.user_cloud.network.subnets(network_id=self.EXT_NET_ID):
            self.EXT_SUB_ID = sub.id

        # Create Internal Network
        net = self._create_network(self.INT_NET_NAME)
        self.INT_NET_ID = net.id
        sub = self._create_subnet(
            self.INT_SUB_NAME, self.INT_NET_ID, self.INT_CIDR
        )
        self.INT_SUB_ID = sub.id

        # Create Router
        sot = self.user_cloud.network.create_router(
            name=self.ROT_NAME,
            **{"external_gateway_info": {"network_id": self.EXT_NET_ID}},
        )
        assert isinstance(sot, router.Router)
        self.assertEqual(self.ROT_NAME, sot.name)
        self.ROT_ID = sot.id
        self.ROT = sot

        # Add Router's Interface to Internal Network
        interface = self.ROT.add_interface(
            self.user_cloud.network, subnet_id=self.INT_SUB_ID
        )
        self.assertEqual(interface["subnet_id"], self.INT_SUB_ID)

        # Create Port in Internal Network
        prt = self.user_cloud.network.create_port(network_id=self.INT_NET_ID)
        assert isinstance(prt, port.Port)
        self.INTERNAL_PORT_ID = prt.id
        self.INTERNAL_IP_ADDRESS = prt.fixed_ips[0]["ip_address"]

        # Create Floating IP.
        fip = self.user_cloud.network.create_ip(
            floating_network_id=self.EXT_NET_ID
        )
        assert isinstance(fip, floating_ip.FloatingIP)
        self.FIP_ID = fip.id

        # Create Port Forwarding
        pf = self.user_cloud.network.create_port_forwarding(
            floatingip_id=self.FIP_ID,
            internal_port_id=self.INTERNAL_PORT_ID,
            internal_ip_address=self.INTERNAL_IP_ADDRESS,
            internal_port=self.INTERNAL_PORT,
            external_port=self.EXTERNAL_PORT,
            protocol=self.PROTOCOL,
            description=self.DESCRIPTION,
        )
        assert isinstance(pf, _port_forwarding.PortForwarding)
        self.PF = pf

    def tearDown(self):
        sot = self.user_cloud.network.delete_port_forwarding(
            self.PF, self.FIP_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.user_cloud.network.delete_ip(
            self.FIP_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.user_cloud.network.delete_port(
            self.INTERNAL_PORT_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.ROT.remove_interface(
            self.user_cloud.network, subnet_id=self.INT_SUB_ID
        )
        self.assertEqual(sot["subnet_id"], self.INT_SUB_ID)
        sot = self.user_cloud.network.delete_router(
            self.ROT_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.user_cloud.network.delete_subnet(
            self.INT_SUB_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.user_cloud.network.delete_network(
            self.INT_NET_ID, ignore_missing=False
        )
        self.assertIsNone(sot)
        super().tearDown()

    def _create_network(self, name, **args):
        self.name = name
        net = self.user_cloud.network.create_network(name=name, **args)
        assert isinstance(net, network.Network)
        self.assertEqual(self.name, net.name)
        return net

    def _create_subnet(self, name, net_id, cidr):
        self.name = name
        self.net_id = net_id
        self.cidr = cidr
        sub = self.user_cloud.network.create_subnet(
            name=self.name,
            ip_version=self.IPV4,
            network_id=self.net_id,
            cidr=self.cidr,
        )
        assert isinstance(sub, subnet.Subnet)
        self.assertEqual(self.name, sub.name)
        return sub

    def test_find(self):
        sot = self.user_cloud.network.find_port_forwarding(
            self.PF.id, self.FIP_ID, ignore_missing=False
        )
        self.assertEqual(self.INTERNAL_PORT_ID, sot.internal_port_id)
        self.assertEqual(self.INTERNAL_IP_ADDRESS, sot.internal_ip_address)
        self.assertEqual(self.INTERNAL_PORT, sot.internal_port)
        self.assertEqual(self.EXTERNAL_PORT, sot.external_port)
        self.assertEqual(self.PROTOCOL, sot.protocol)
        self.assertEqual(self.DESCRIPTION, sot.description)

    def test_get(self):
        sot = self.user_cloud.network.get_port_forwarding(self.PF, self.FIP_ID)
        self.assertEqual(self.INTERNAL_PORT_ID, sot.internal_port_id)
        self.assertEqual(self.INTERNAL_IP_ADDRESS, sot.internal_ip_address)
        self.assertEqual(self.INTERNAL_PORT, sot.internal_port)
        self.assertEqual(self.EXTERNAL_PORT, sot.external_port)
        self.assertEqual(self.PROTOCOL, sot.protocol)
        self.assertEqual(self.DESCRIPTION, sot.description)

    def test_list(self):
        pf_ids = [
            o.id for o in self.user_cloud.network.port_forwardings(self.FIP_ID)
        ]
        self.assertIn(self.PF.id, pf_ids)

    def test_update(self):
        NEW_EXTERNAL_PORT = 90
        sot = self.user_cloud.network.update_port_forwarding(
            self.PF.id, self.FIP_ID, external_port=NEW_EXTERNAL_PORT
        )
        self.assertEqual(NEW_EXTERNAL_PORT, sot.external_port)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_keypairs
----------------------------------

Functional tests for keypairs methods
"""

from openstack.tests.functional import base

FAKE_PUBLIC_KEY = (
    "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCkF3MX59OrlBs3dH5CU7lNmvpbrgZxSpyGj"
    "lnE8Flkirnc/Up22lpjznoxqeoTAwTW034k7Dz6aYIrZGmQwe2TkE084yqvlj45Dkyoj95fW/"
    "sZacm0cZNuL69EObEGHdprfGJQajrpz22NQoCD8TFB8Wv+8om9NH9Le6s+WPe98WC77KLw8qg"
    "fQsbIey+JawPWl4O67ZdL5xrypuRjfIPWjgy/VH85IXg/Z/GONZ2nxHgSShMkwqSFECAC5L3P"
    "HB+0+/12M/iikdatFSVGjpuHvkLOs3oe7m6HlOfluSJ85BzLWBbvva93qkGmLg4ZAc8rPh2O+"
    "YIsBUHNLLMM/oQp Generated-by-Nova\n"
)


class TestKeypairs(base.BaseFunctionalTest):
    def test_create_and_delete(self):
        '''Test creating and deleting keypairs functionality'''
        name = self.getUniqueString('keypair')
        self.addCleanup(self.user_cloud.delete_keypair, name)
        keypair = self.user_cloud.create_keypair(name=name)
        self.assertEqual(keypair['name'], name)
        self.assertIsNotNone(keypair['public_key'])
        self.assertIsNotNone(keypair['private_key'])
        self.assertIsNotNone(keypair['fingerprint'])
        self.assertEqual(keypair['type'], 'ssh')

        keypairs = self.user_cloud.list_keypairs()
        self.assertIn(name, [k['name'] for k in keypairs])

        self.user_cloud.delete_keypair(name)

        keypairs = self.user_cloud.list_keypairs()
        self.assertNotIn(name, [k['name'] for k in keypairs])

    def test_create_and_delete_with_key(self):
        '''Test creating and deleting keypairs functionality'''
        name = self.getUniqueString('keypair')
        self.addCleanup(self.user_cloud.delete_keypair, name)
        keypair = self.user_cloud.create_keypair(
            name=name, public_key=FAKE_PUBLIC_KEY
        )
        self.assertEqual(keypair['name'], name)
        self.assertIsNotNone(keypair['public_key'])
        self.assertIsNone(keypair['private_key'])
        self.assertIsNotNone(keypair['fingerprint'])
        self.assertEqual(keypair['type'], 'ssh')

        keypairs = self.user_cloud.list_keypairs()
        self.assertIn(name, [k['name'] for k in keypairs])

        self.user_cloud.delete_keypair(name)

        keypairs = self.user_cloud.list_keypairs()
        self.assertNotIn(name, [k['name'] for k in keypairs])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import _base
from openstack import resource


class Network(_base.NetworkResource, _base.TagMixinNetwork):
    resource_key = 'network'
    resources_key = 'networks'
    base_path = '/networks'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # NOTE: We don't support query on list or datetime fields yet
    _query_mapping = resource.QueryParameters(
        'description',
        'name',
        'status',
        'project_id',
        'sort_key',
        'sort_dir',
        'id',
        ipv4_address_scope_id='ipv4_address_scope',
        ipv6_address_scope_id='ipv6_address_scope',
        is_admin_state_up='admin_state_up',
        is_port_security_enabled='port_security_enabled',
        is_router_external='router:external',
        is_shared='shared',
        provider_network_type='provider:network_type',
        provider_physical_network='provider:physical_network',
        provider_segmentation_id='provider:segmentation_id',
        **_base.TagMixinNetwork._tag_query_parameters,
    )

    # Properties
    #: Availability zone hints to use when scheduling the network.
    #: *Type: list of availability zone names*
    availability_zone_hints = resource.Body(
        'availability_zone_hints', type=list
    )
    #: Availability zones for the network.
    #: *Type: list of availability zone names*
    availability_zones = resource.Body('availability_zones', type=list)
    #: Timestamp when the network was created.
    created_at = resource.Body('created_at')
    #: The network description.
    description = resource.Body('description')
    #: The DNS domain associated.
    dns_domain = resource.Body('dns_domain')
    #: The ID of the IPv4 address scope for the network.
    ipv4_address_scope_id = resource.Body('ipv4_address_scope')
    #: The ID of the IPv6 address scope for the network.
    ipv6_address_scope_id = resource.Body('ipv6_address_scope')
    #: The administrative state of the network, which is up ``True`` or
    #: down ``False``. *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Whether or not this is the default external network.
    #: *Type: bool*
    is_default = resource.Body('is_default', type=bool)
    #: The port security status, which is enabled ``True`` or disabled
    #: ``False``. *Type: bool* *Default: False*
    #: Available for multiple provider extensions.
    is_port_security_enabled = resource.Body(
        'port_security_enabled', type=bool, default=False
    )
    #: Whether or not the router is external.
    #: *Type: bool* *Default: False*
    is_router_external = resource.Body(
        'router:external', type=bool, default=False
    )
    #: Indicates whether this network is shared across all projects.
    #: By default, only administrative users can change this value.
    #: *Type: bool*
    is_shared = resource.Body('shared', type=bool)
    #: Read-only. The maximum transmission unit (MTU) of the network resource.
    mtu = resource.Body('mtu', type=int)
    #: The network name.
    name = resource.Body('name')
    #: The ID of the project this network is associated with.
    project_id = resource.Body('project_id')
    #: The type of physical network that maps to this network resource.
    #: For example, ``flat``, ``vlan``, ``vxlan``, or ``gre``.
    #: Available for multiple provider extensions.
    provider_network_type = resource.Body('provider:network_type')
    #: The physical network where this network object is implemented.
    #: Available for multiple provider extensions.
    provider_physical_network = resource.Body('provider:physical_network')
    #: An isolated segment ID on the physical network. The provider
    #: network type defines the segmentation model.
    #: Available for multiple provider extensions.
    provider_segmentation_id = resource.Body('provider:segmentation_id')
    #: Indicates if PVLAN is enabled for the network.
    pvlan = resource.Body('pvlan', type=bool)
    #: The ID of the QoS policy attached to the port.
    qos_policy_id = resource.Body('qos_policy_id')
    #: A list of provider segment objects.
    #: Available for multiple provider extensions.
    segments = resource.Body('segments', type=list)
    #: The network status.
    status = resource.Body('status')
    #: The associated subnet IDs.
    #: *Type: list of strs of the subnet IDs*
    subnet_ids = resource.Body('subnets', type=list)
    #: Timestamp when the network was last updated.
    updated_at = resource.Body('updated_at')
    #: Indicates the VLAN transparency mode of the network
    is_vlan_transparent = resource.Body('vlan_transparent', type=bool)
    #: Indicates the VLAN QinQ mode of the network
    is_vlan_qinq = resource.Body('vlan_qinq', type=bool)


class DHCPAgentHostingNetwork(Network):
    resource_key = 'network'
    resources_key = 'networks'
    base_path = '/agents/%(agent_id)s/dhcp-networks'
    resource_name = 'dhcp-network'

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    # NOTE: No query parameter is supported



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import manageable_snapshot
from openstack.tests.unit import base

EXAMPLE = {
    'reference': {'source-name': 'snapshot-00000001'},
    'source_reference': {'source-name': 'volume-00000001'},
    'size': 1,
    'safe_to_manage': True,
    'reason_not_safe': None,
    'cinder_id': None,
    'extra_info': None,
}


class TestManageableSnapshot(base.TestCase):
    def test_basic(self):
        sot = manageable_snapshot.ManageableSnapshot()
        self.assertEqual('manageable-snapshot', sot.resource_key)
        self.assertEqual('manageable-snapshots', sot.resources_key)
        self.assertEqual('/manageable_snapshots', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

        self.assertDictEqual(
            {
                'cluster': 'cluster',
                'host': 'host',
                'limit': 'limit',
                'marker': 'marker',
                'offset': 'offset',
                'sort': 'sort',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = manageable_snapshot.ManageableSnapshot(**EXAMPLE)
        self.assertEqual(EXAMPLE['reference'], sot.reference)
        self.assertEqual(EXAMPLE['source_reference'], sot.source_reference)
        self.assertEqual(EXAMPLE['size'], sot.size)
        self.assertTrue(sot.safe_to_manage)
        self.assertIsNone(sot.reason_not_safe)
        self.assertIsNone(sot.cinder_id)
        self.assertIsNone(sot.extra_info)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from openstack.clustering.v1 import profile_type
from openstack.tests.unit import base

FAKE = {
    'name': 'FAKE_PROFILE_TYPE',
    'schema': {'foo': 'bar'},
    'support_status': {
        '1.0': [
            {
                'status': 'supported',
                'since': '2016.10',
            }
        ]
    },
}


class TestProfileType(base.TestCase):
    def test_basic(self):
        sot = profile_type.ProfileType()
        self.assertEqual('profile_type', sot.resource_key)
        self.assertEqual('profile_types', sot.resources_key)
        self.assertEqual('/profile-types', sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_list)

    def test_instantiate(self):
        sot = profile_type.ProfileType(**FAKE)
        self.assertEqual(FAKE['name'], sot._get_id(sot))
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['schema'], sot.schema)
        self.assertEqual(FAKE['support_status'], sot.support_status)

    def test_ops(self):
        sot = profile_type.ProfileType(**FAKE)

        resp = mock.Mock()
        resp.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.get = mock.Mock(return_value=resp)
        self.assertEqual('', sot.type_ops(sess))
        url = f'profile-types/{sot.id}/ops'
        sess.get.assert_called_once_with(url)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import service_profile as _service_profile
from openstack.tests.functional import base


class TestServiceProfile(base.BaseFunctionalTest):
    SERVICE_PROFILE_DESCRIPTION = "DESCRIPTION"
    UPDATE_DESCRIPTION = "UPDATED-DESCRIPTION"
    METAINFO = "FlAVOR_PROFILE_METAINFO"
    ID: str | None = None

    def setUp(self):
        super().setUp()
        if not self.user_cloud._has_neutron_extension("flavors"):
            self.skipTest("Neutron flavor extension is required for this test")

        if self.operator_cloud:
            service_profiles = (
                self.operator_cloud.network.create_service_profile(
                    description=self.SERVICE_PROFILE_DESCRIPTION,
                    metainfo=self.METAINFO,
                )
            )
            assert isinstance(
                service_profiles, _service_profile.ServiceProfile
            )
            self.assertEqual(
                self.SERVICE_PROFILE_DESCRIPTION, service_profiles.description
            )
            self.assertEqual(self.METAINFO, service_profiles.meta_info)

            self.ID = service_profiles.id

    def tearDown(self):
        if self.ID:
            service_profiles = (
                self.operator_cloud.network.delete_service_profile(
                    self.ID, ignore_missing=True
                )
            )
            self.assertIsNone(service_profiles)
        super().tearDown()

    def test_find(self):
        self.user_cloud.network.find_service_profile(
            name_or_id="not_existing", ignore_missing=True
        )
        if self.operator_cloud and self.ID:
            service_profiles = (
                self.operator_cloud.network.find_service_profile(
                    self.ID, ignore_missing=False
                )
            )
            self.assertEqual(self.METAINFO, service_profiles.meta_info)

    def test_get(self):
        if not self.ID:
            self.skipTest("ServiceProfile was not created")
        service_profiles = self.operator_cloud.network.get_service_profile(
            self.ID
        )
        self.assertEqual(self.METAINFO, service_profiles.meta_info)
        self.assertEqual(
            self.SERVICE_PROFILE_DESCRIPTION, service_profiles.description
        )

    def test_update(self):
        if not self.ID:
            self.skipTest("ServiceProfile was not created")
        service_profiles = self.operator_cloud.network.update_service_profile(
            self.ID, description=self.UPDATE_DESCRIPTION
        )
        self.assertEqual(self.UPDATE_DESCRIPTION, service_profiles.description)

    def test_list(self):
        # Test in user scope
        self.user_cloud.network.service_profiles()
        # Test as operator
        if self.operator_cloud:
            metainfos = [
                f.meta_info
                for f in self.operator_cloud.network.service_profiles()
            ]
            if self.ID:
                self.assertIn(self.METAINFO, metainfos)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import agent
from openstack.network.v2 import network
from openstack.tests.functional import base


class TestAgentNetworks(base.BaseFunctionalTest):
    NETWORK_ID: str
    AGENT: agent.Agent
    AGENT_ID: str

    def setUp(self):
        super().setUp()
        if not self.user_cloud._has_neutron_extension("agent"):
            self.skipTest("Neutron agent extension is required for this test")

        self.NETWORK_NAME = self.getUniqueString("network")
        net = self.user_cloud.network.create_network(name=self.NETWORK_NAME)
        self.addCleanup(self.user_cloud.network.delete_network, net.id)
        assert isinstance(net, network.Network)
        self.NETWORK_ID = net.id
        agent_list = list(self.user_cloud.network.agents())
        agents = [
            agent for agent in agent_list if agent.agent_type == "DHCP agent"
        ]
        if len(agent_list) == 0:
            self.skipTest("No agents available")

        self.AGENT = agents[0]
        self.AGENT_ID = self.AGENT.id

    def test_add_remove_agent(self):
        self.AGENT.add_agent_to_network(
            self.user_cloud.network, network_id=self.NETWORK_ID
        )
        self._verify_add()

        self.AGENT.remove_agent_from_network(
            self.user_cloud.network, network_id=self.NETWORK_ID
        )
        self._verify_remove()

    def _verify_add(self):
        net = self.user_cloud.network.dhcp_agent_hosting_networks(
            self.AGENT_ID
        )
        net_ids = [n.id for n in net]
        self.assertIn(self.NETWORK_ID, net_ids)

    def _verify_remove(self):
        net = self.user_cloud.network.dhcp_agent_hosting_networks(
            self.AGENT_ID
        )
        net_ids = [n.id for n in net]
        self.assertNotIn(self.NETWORK_ID, net_ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any

import requests

from keystoneauth1 import adapter

from openstack.clustering.v1 import action as _action
from openstack import exceptions
from openstack import resource
from openstack import utils


class AsyncResource(resource.Resource):
    def delete(  # type: ignore[override]
        self,
        session: adapter.Adapter,
        error_message: str | None = None,
        *,
        microversion: str | None = None,
        **kwargs: Any,
    ) -> _action.Action:
        """Delete the remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: An :class:`~openstack.clustering.v1.action.Action`
                 instance. The ``fetch`` method will need to be used
                 to populate the `Action` with status information.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_commit` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.NotFoundException` if
                 the resource was not found.
        """
        response = self._raw_delete(session)
        return self._delete_response(response, error_message)

    def _delete_response(
        self,
        response: requests.Response,
        error_message: str | None = None,
    ) -> _action.Action:
        exceptions.raise_from_response(response, error_message=error_message)
        location = response.headers['Location']
        action_id = location.split('/')[-1]
        action = _action.Action.existing(
            id=action_id, connection=self._connection
        )
        return action

    def force_delete(self, session: adapter.Adapter) -> _action.Action:
        """Force delete the remote resource."""
        body = {'force': True}
        url = utils.urljoin(self.base_path, self.id)
        response = session.delete(url, json=body)
        return self._delete_response(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any, Self

from keystoneauth1 import adapter
import requests

from openstack.common import metadata
from openstack import exceptions
from openstack import resource
from openstack import utils


class Share(resource.Resource, metadata.MetadataMixin):
    resource_key = "share"
    resources_key = "shares"
    base_path = "/shares"

    _query_mapping = resource.QueryParameters(
        "project_id",
        "name",
        "status",
        "share_server_id",
        "metadata",
        "share_type_id",
        "snapshot_id",
        "host",
        "share_network_id",
        "is_public",
        "share_group_id",
        "export_location_id",
        "export_location_path",
        "limit",
        "offset",
        "sort_key",
        "sort_dir",
        all_projects="all_tenants",
    )

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_list = True
    allow_head = False
    allow_delete = True

    #: Properties
    #: The share instance access rules status. A valid value is active,
    #: error, or syncing.
    access_rules_status = resource.Body("access_rules_status", type=str)
    #: The availability zone.
    availability_zone = resource.Body("availability_zone", type=str)
    #: The date and time stamp when the resource was created within the
    #: services's database.
    created_at = resource.Body("created_at", type=str)
    #: The user defined description of the resource.
    description = resource.Body("description", type=str)
    #: The share host name.
    host = resource.Body("host", type=str)
    #: The level of visibility for the share.
    is_public = resource.Body("is_public", type=bool)
    #: Whether or not this share supports snapshots that can be
    #: cloned into new shares.
    is_creating_new_share_from_snapshot_supported = resource.Body(
        "create_share_from_snapshot_support", type=bool
    )
    #: Whether the share's snapshots can be mounted directly and access
    #: controlled independently or not.
    is_mounting_snapshot_supported = resource.Body(
        "mount_snapshot_support", type=bool
    )
    #: Whether the share can be reverted to its latest snapshot or not.
    is_reverting_to_snapshot_supported = resource.Body(
        "revert_to_snapshot_support", type=bool
    )
    #: An extra specification that filters back ends by whether the share
    #: supports snapshots or not.
    is_snapshot_supported = resource.Body("snapshot_support", type=bool)
    #: Indicates whether the share has replicas or not.
    is_replicated = resource.Body("has_replicas", type=bool)
    #: One or more metadata key and value pairs as a dictionary of strings.
    metadata = resource.Body("metadata", type=dict)
    #: The progress of the share creation.
    progress = resource.Body("progress", type=str)
    #: The ID of the project that owns the resource.
    project_id = resource.Body("project_id", type=str)
    #: The share replication type. Valid values are none, readable,
    #: writable and dr.
    replication_type = resource.Body("replication_type", type=str)
    #: The UUID of the share group that this shares belongs to.
    share_group_id = resource.Body("share_group_id", type=str)
    #: The share network ID.
    share_network_id = resource.Body("share_network_id", type=str)
    #: The Shared File Systems protocol. A valid value is NFS,
    #: CIFS, GlusterFS, HDFS, CephFS, MAPRFS
    share_protocol = resource.Body("share_proto", type=str)
    #: The UUID of the share server.
    share_server_id = resource.Body("share_server_id", type=str)
    #: The UUID of the share type. In minor versions, this parameter is a
    #: share type name, as a string.
    share_type = resource.Body("share_type", type=str)
    #: Name of the share type.
    share_type_name = resource.Body("share_type_name", type=str)
    #: The share size, in GiBs.
    size = resource.Body("size", type=int)
    #: The UUID of the snapshot that was used to create the
    #: share.
    snapshot_id = resource.Body("snapshot_id", type=str)
    #: The ID of the group snapshot instance that was used to create
    #: this share.
    source_share_group_snapshot_member_id = resource.Body(
        "source_share_group_snapshot_member_id", type=str
    )
    #: The share status
    status = resource.Body("status", type=str)
    #: For the share migration, the migration task state.
    task_state = resource.Body("task_state", type=str)
    #: ID of the user that the share was created by.
    user_id = resource.Body("user_id", type=str)
    #: Display name for updating name
    display_name = resource.Body("display_name", type=str)
    #: Display description for updating description
    display_description = resource.Body("display_description", type=str)

    def _action(
        self,
        session: adapter.Adapter,
        body: dict[str, Any],
        microversion: str | None = None,
    ) -> requests.Response:
        """Perform share instance actions given the message body"""
        url = utils.urljoin(self.base_path, self.id, 'action')
        headers = {'Accept': ''}

        if microversion is None:
            microversion = self._get_microversion(session)

        response = session.post(
            url, json=body, headers=headers, microversion=microversion
        )

        exceptions.raise_from_response(response)
        return response

    def extend_share(
        self, session: adapter.Adapter, new_size: float, force: bool = False
    ) -> None:
        """Extend the share size.

        :param session: A session object used for sending request.
        :param new_size: The new size of the share in GiB.
        :param force: Whether or not to use force, bypassing the scheduler.
            Requires admin privileges. Defaults to False.
        :returns: The result of the action.
        """

        extend_body = {"new_size": new_size}

        if force is True:
            extend_body['force'] = True

        body = {"extend": extend_body}
        self._action(session, body)

    def shrink_share(self, session: adapter.Adapter, new_size: float) -> None:
        """Shrink the share size.

        :param session: A session object used for sending request.
        :param new_size: The new size of the share in GiB.
        :returns: ``None``
        """

        body = {"shrink": {'new_size': new_size}}
        self._action(session, body)

    def revert_to_snapshot(
        self, session: adapter.Adapter, snapshot_id: str
    ) -> None:
        """Revert the share to the given snapshot.

        :param session: A session object used for sending request.
        :param snapshot_id: The id of the snapshot to revert to.
        :returns: ``None``
        """
        body = {"revert": {"snapshot_id": snapshot_id}}
        self._action(session, body)

    def manage(
        self,
        session: adapter.Adapter,
        protocol: str,
        export_path: str,
        service_host: str,
        **params: Any,
    ) -> Self:
        """Manage a share.

        :param session: A session object used for sending request.
        :param protocol: The shared file systems protocol of this share.
        :param export_path: The export path formatted according to the
            protocol.
        :param service_host: The manage-share service host.
        :param params: Optional parameters to be sent. Available
            parameters include:

            * name: The user defined name of the resource.
            * share_type: The name or ID of the share type to be used to create
              the resource.
            * driver_options: A set of one or more key and value pairs, as a
              dictionary of strings, that describe driver options.
            * is_public: The level of visibility for the share.
            * description: The user defiend description of the resource.
            * share_server_id: The UUID of the share server.

        :returns: The share that was managed.
        """
        attrs = {
            'share': {
                'protocol': protocol,
                'export_path': export_path,
                'service_host': service_host,
            }
        }

        attrs['share'].update(params)

        url = utils.urljoin(self.base_path, 'manage')
        resp = session.post(url, json=attrs)
        self._translate_response(resp)
        return self

    def unmanage(self, session: adapter.Adapter) -> None:
        """Unmanage a share.

        :param session: A session object used for sending request.
        :returns: ``None``
        """
        body = {'unmanage': None}
        self._action(session, body)

    def force_delete(self, session: adapter.Adapter) -> None:
        """Force delete the share.

        :returns: ``None``
        """
        body = {"force_delete": None}
        self._action(session, body)

    def soft_delete(self, session: adapter.Adapter) -> None:
        """Soft delete a share.

        :param session: A session object used for sending request.
        :returns: ``None``
        """
        body = {'soft_delete': None}
        self._action(session, body)

    def restore(self, session: adapter.Adapter) -> None:
        """Restore a share.

        :param session: A session object used for sending request.
        :returns: ``None``
        """
        body = {'restore': None}
        self._action(session, body)

    def reset_status(self, session: adapter.Adapter, status: str) -> None:
        """Reset the share to the given status.

        :param status: The status of the share to reset to.
        :returns: ``None``
        """
        body = {"reset_status": {"status": status}}
        self._action(session, body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.compute.v2 import server_share
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "uuid": "715335c1-7a00-4dfe-82df-9dc2a67bd8bf",
    "share_id": "e8debdc0-447a-4376-a10a-4cd9122d7986",
    "share_proto": "NFS",
    "status": "active",
    "tag": "bar",
    "export_location": "server.com/nfs_mount,foo=bar",
}


class TestServerShare(base.TestCase):
    def test_basic(self):
        sot = server_share.ShareMapping()
        self.assertEqual('share', sot.resource_key)
        self.assertEqual('shares', sot.resources_key)
        self.assertEqual('/servers/%(server_id)s/shares', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = server_share.ShareMapping(**EXAMPLE)
        self.assertEqual(EXAMPLE['uuid'], sot.uuid)
        self.assertEqual(EXAMPLE['share_id'], sot.share_id)
        self.assertEqual(EXAMPLE['share_id'], sot.id)
        self.assertEqual(EXAMPLE['share_proto'], sot.share_proto)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['tag'], sot.tag)
        self.assertEqual(EXAMPLE['export_location'], sot.export_location)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import bgpvpn as _bgpvpn
from openstack.network.v2 import (
    bgpvpn_network_association as _bgpvpn_net_assoc,
)
from openstack.network.v2 import bgpvpn_port_association as _bgpvpn_port_assoc
from openstack.network.v2 import (
    bgpvpn_router_association as _bgpvpn_router_assoc,
)
from openstack.network.v2 import network as _network
from openstack.network.v2 import port as _port
from openstack.network.v2 import router as _router
from openstack.network.v2 import subnet as _subnet
from openstack.tests.functional import base


class TestBGPVPN(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()

        self.BGPVPN_NAME = 'my_bgpvpn' + self.getUniqueString()
        self.NET_NAME = 'my_net' + self.getUniqueString()
        self.SUBNET_NAME = 'my_subnet' + self.getUniqueString()
        self.PORT_NAME = 'my_port' + self.getUniqueString()
        self.ROUTER_NAME = 'my_router' + self.getUniqueString()
        self.CIDR = "10.101.0.0/24"
        self.ROUTE_DISTINGUISHERS = ['64512:1777', '64512:1888', '64512:1999']
        self.VNI = 1000
        self.ROUTE_TARGETS = ('64512:1444',)
        self.IMPORT_TARGETS = ('64512:1555',)
        self.EXPORT_TARGETS = '64512:1666'
        self.TYPE = 'l3'

        if not self.user_cloud.network.find_extension("bgpvpn"):
            self.skipTest("Neutron BGPVPN Extension disabled")
        bgpvpn = self.operator_cloud.network.create_bgpvpn(
            name=self.BGPVPN_NAME,
            route_distinguishers=self.ROUTE_DISTINGUISHERS,
            route_targets=self.ROUTE_TARGETS,
            import_targets=self.IMPORT_TARGETS,
            export_targets=self.EXPORT_TARGETS,
        )
        assert isinstance(bgpvpn, _bgpvpn.BgpVpn)
        self.BGPVPN = bgpvpn

        net = self.operator_cloud.network.create_network(name=self.NET_NAME)
        assert isinstance(net, _network.Network)
        self.NETWORK = net
        subnet = self.operator_cloud.network.create_subnet(
            name=self.SUBNET_NAME,
            ip_version=4,
            network_id=self.NETWORK.id,
            cidr=self.CIDR,
        )
        assert isinstance(subnet, _subnet.Subnet)
        self.SUBNET = subnet

        port = self.operator_cloud.network.create_port(
            name=self.PORT_NAME, network_id=self.NETWORK.id
        )
        assert isinstance(port, _port.Port)
        self.PORT = port

        router = self.operator_cloud.network.create_router(
            name=self.ROUTER_NAME
        )
        assert isinstance(router, _router.Router)
        self.ROUTER = router

        net_assoc = (
            self.operator_cloud.network.create_bgpvpn_network_association(
                self.BGPVPN, network_id=self.NETWORK.id
            )
        )
        assert isinstance(
            net_assoc, _bgpvpn_net_assoc.BgpVpnNetworkAssociation
        )
        self.NET_ASSOC = net_assoc

        port_assoc = (
            self.operator_cloud.network.create_bgpvpn_port_association(
                self.BGPVPN, port_id=self.PORT.id
            )
        )
        assert isinstance(port_assoc, _bgpvpn_port_assoc.BgpVpnPortAssociation)
        self.PORT_ASSOC = port_assoc

        router_assoc = (
            self.operator_cloud.network.create_bgpvpn_router_association(
                self.BGPVPN, router_id=self.ROUTER.id
            )
        )
        assert isinstance(
            router_assoc, _bgpvpn_router_assoc.BgpVpnRouterAssociation
        )
        self.ROUTER_ASSOC = router_assoc

    def tearDown(self):
        sot = self.operator_cloud.network.delete_bgpvpn(self.BGPVPN.id)
        self.assertIsNone(sot)
        sot = self.operator_cloud.network.delete_bgpvpn_network_association(
            self.BGPVPN.id, self.NET_ASSOC.id
        )
        self.assertIsNone(sot)

        sot = self.operator_cloud.network.delete_bgpvpn_port_association(
            self.BGPVPN.id, self.PORT_ASSOC.id
        )
        self.assertIsNone(sot)
        sot = self.operator_cloud.network.delete_bgpvpn_router_association(
            self.BGPVPN.id, self.ROUTER_ASSOC.id
        )
        self.assertIsNone(sot)

        sot = self.operator_cloud.network.delete_router(self.ROUTER)
        self.assertIsNone(sot)
        sot = self.operator_cloud.network.delete_port(self.PORT)
        self.assertIsNone(sot)
        sot = self.operator_cloud.network.delete_subnet(self.SUBNET)
        self.assertIsNone(sot)
        sot = self.operator_cloud.network.delete_network(self.NETWORK)
        self.assertIsNone(sot)

        super().tearDown()

    def test_find_bgpvpn(self):
        sot = self.operator_cloud.network.find_bgpvpn(
            self.BGPVPN.name, ignore_missing=False
        )
        self.assertEqual(list(self.ROUTE_TARGETS), sot.route_targets)
        self.assertEqual(list(self.IMPORT_TARGETS), sot.import_targets)
        # Check defaults
        self.assertEqual(self.TYPE, sot.type)

    def test_get_bgpvpn(self):
        sot = self.operator_cloud.network.get_bgpvpn(self.BGPVPN.id)
        self.assertEqual(list(self.ROUTE_TARGETS), sot.route_targets)
        self.assertEqual([self.EXPORT_TARGETS], sot.export_targets)
        self.assertEqual(list(self.IMPORT_TARGETS), sot.import_targets)

    def test_list_bgpvpns(self):
        bgpvpn_ids = [
            bgpvpn.id for bgpvpn in self.operator_cloud.network.bgpvpns()
        ]
        self.assertIn(self.BGPVPN.id, bgpvpn_ids)

    def test_update_bgpvpn(self):
        sot = self.operator_cloud.network.update_bgpvpn(
            self.BGPVPN.id, import_targets='64512:1333'
        )
        self.assertEqual(['64512:1333'], sot.import_targets)

    def test_get_bgpvpnnetwork_association(self):
        sot = self.operator_cloud.network.get_bgpvpn_network_association(
            self.BGPVPN.id, self.NET_ASSOC.id
        )
        self.assertEqual(self.NETWORK.id, sot.network_id)

    def test_list_bgpvpn_network_associations(self):
        net_assoc_ids = [
            net_assoc.id
            for net_assoc in (
                self.operator_cloud.network.bgpvpn_network_associations(
                    self.BGPVPN.id
                )
            )
        ]
        self.assertIn(self.NET_ASSOC.id, net_assoc_ids)

    def test_get_bgpvpn_port_association(self):
        sot = self.operator_cloud.network.get_bgpvpn_port_association(
            self.BGPVPN.id, self.PORT_ASSOC.id
        )
        self.assertEqual(self.PORT.id, sot.port_id)

    def test_list_bgpvpn_port_associations(self):
        port_assoc_ids = [
            port_assoc.id
            for port_assoc in (
                self.operator_cloud.network.bgpvpn_port_associations(
                    self.BGPVPN.id
                )
            )
        ]
        self.assertIn(self.PORT_ASSOC.id, port_assoc_ids)

    def test_get_bgpvpn_router_association(self):
        sot = self.operator_cloud.network.get_bgpvpn_router_association(
            self.BGPVPN.id, self.ROUTER_ASSOC.id
        )
        self.assertEqual(self.ROUTER.id, sot.router_id)

    def test_list_bgpvpn_router_associations(self):
        router_assoc_ids = [
            router_assoc.id
            for router_assoc in (
                self.operator_cloud.network.bgpvpn_router_associations(
                    self.BGPVPN.id
                )
            )
        ]
        self.assertIn(self.ROUTER_ASSOC.id, router_assoc_ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Create resources from the DNS service.

For a full guide see
https://docs.openstack.org/openstacksdk/latest/user/guides/dns.html
"""


def create_zone(
    conn,
    name,
    email,
    ttl=3600,
    description="Default description",
    zone_type="PRIMARY",
):
    print("Create Zone: ")

    zone = {
        "name": name,
        "email": email,
        "ttl": ttl,
        "description": description,
        "type": zone_type,
    }

    print(conn.dns.create_zone(**zone))


def create_recordset(
    conn,
    name_or_id,
    recordset_name,
    recordset_type="A",
    records=["192.168.1.1"],
    ttl=3600,
    description="Default description",
):
    print("Create Recordset: ")

    zone = conn.dns.find_zone(name_or_id)

    if not zone:
        print("Zone not found.")
        return None

    zone_id = zone.id

    recordset_data = {
        "name": recordset_name,
        "type": recordset_type,
        "records": records,
        "ttl": ttl,
        "description": description,
    }

    print(conn.dns.create_recordset(zone_id, **recordset_data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any, Generic, TypeVar

_T = TypeVar('_T')


class Formatter(Generic[_T]):
    @classmethod
    def deserialize(cls, value: Any) -> _T:
        """Return a formatted object representing the value"""
        raise NotImplementedError


class BoolStr(Formatter[bool]):
    @classmethod
    def deserialize(cls, value: Any) -> bool:
        """Convert a boolean string to a boolean"""
        expr = str(value).lower()
        if "true" == expr:
            return True
        elif "false" == expr:
            return False
        else:
            raise ValueError(f"Unable to deserialize boolean string: {value}")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions as sdk_exc
from openstack.identity.v3 import _proxy as _identity_v3
from openstack.key_manager.v1 import project_quota as _project_quota
from openstack.tests.functional import base

# NOTE(jbeen): Barbican policy may require 'key-manager:service-admin' for
# project quotas. Create and assign it per test project to avoid 403 errors.
ADMIN_ROLE_NAME = 'key-manager:service-admin'


class TestProjectQuota(base.BaseFunctionalTest):
    _identity: _identity_v3.Proxy

    def setUp(self):
        super().setUp()
        self.require_service('key-manager')

        identity = self.system_admin_cloud.identity
        assert identity.api_version == '3'
        self._identity = identity

        self.project_name = self.getUniqueString('project')
        self.project = self._identity.create_project(
            name=self.project_name,
        )
        self.addCleanup(self._identity.delete_project, self.project)

        self.role = self._identity.create_role(name=ADMIN_ROLE_NAME)
        self.addCleanup(self._identity.delete_role, self.role.id)

        self.user_id = self.system_admin_cloud.current_user_id
        assert self.user_id is not None
        self._identity.assign_project_role_to_user(
            project=self.project, user=self.user_id, role=self.role
        )
        self.addCleanup(
            self._identity.unassign_project_role_from_user,
            project=self.project,
            user=self.user_id,
            role=self.role,
        )

        self._set_operator_cloud(project_id=self.project.id)

    def test_project_quotas(self):
        # update project quota
        project_quota = self.operator_cloud.key_manager.update_project_quota(
            self.project.id,
            secrets=1,
            orders=2,
            containers=3,
            consumers=4,
            cas=5,
        )

        self.assertIsInstance(project_quota, _project_quota.ProjectQuota)
        self.assertIsNotNone(project_quota.id)
        self.assertEqual(1, project_quota.secrets)
        self.assertEqual(2, project_quota.orders)
        self.assertEqual(3, project_quota.containers)
        self.assertEqual(4, project_quota.consumers)
        self.assertEqual(5, project_quota.cas)

        # get project quota
        project_id = self.project.id
        project_quota = self.operator_cloud.key_manager.get_project_quota(
            project_id
        )
        self.assertIsInstance(project_quota, _project_quota.ProjectQuota)

        # delete project quota
        self.operator_cloud.key_manager.delete_project_quota(self.project.id)
        self.assertRaises(
            sdk_exc.NotFoundException,
            self.operator_cloud.key_manager.get_project_quota,
            project_quota,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack import exceptions
from openstack.tests.functional import base
from openstack import utils


class TestZoneShare(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()
        self.require_service('dns')
        if not self.user_cloud:
            self.skipTest("The demo cloud is required for this test")

        # Note: zone deletion is not an immediate operation, so each time
        # chose a new zone name for a test
        # getUniqueString is not guaranteed to return unique string between
        # different tests of the same class.
        self.ZONE_NAME = f'example-{uuid.uuid4().hex}.org.'

        # Make sure the API under test has shared zones support
        if not utils.supports_version(self.operator_cloud.dns, '2.1'):
            self.skipTest(
                'Designate API version does not support shared zones.'
            )

        self.zone = self.operator_cloud.dns.create_zone(
            name=self.ZONE_NAME,
            email='joe@example.org',
            type='PRIMARY',
            ttl=7200,
            description='example zone for sdk zone share tests',
        )
        self.addCleanup(
            self.operator_cloud.dns.delete_zone,
            self.zone,
            delete_shares=True,
        )

        self.project_id = self.operator_cloud.session.get_project_id()
        self.demo_project_id = self.user_cloud.session.get_project_id()

    def test_create_delete_zone_share(self):
        zone_share = self.operator_cloud.dns.create_zone_share(
            self.zone, target_project_id=self.demo_project_id
        )
        self.addCleanup(
            self.operator_cloud.dns.delete_zone_share,
            self.zone,
            zone_share,
        )

        self.assertEqual(self.zone.id, zone_share.zone_id)
        self.assertEqual(self.project_id, zone_share.project_id)
        self.assertEqual(self.demo_project_id, zone_share.target_project_id)
        self.assertIsNotNone(zone_share.id)
        self.assertIsNotNone(zone_share.created_at)
        self.assertIsNone(zone_share.updated_at)

    def test_get_zone_share(self):
        orig_zone_share = self.operator_cloud.dns.create_zone_share(
            self.zone,
            target_project_id=self.demo_project_id,
        )
        self.addCleanup(
            self.operator_cloud.dns.delete_zone_share,
            self.zone,
            orig_zone_share,
        )

        zone_share = self.operator_cloud.dns.get_zone_share(
            self.zone,
            orig_zone_share,
        )

        self.assertEqual(self.zone.id, zone_share.zone_id)
        self.assertEqual(self.project_id, zone_share.project_id)
        self.assertEqual(self.demo_project_id, zone_share.target_project_id)
        self.assertEqual(orig_zone_share.id, zone_share.id)
        self.assertEqual(orig_zone_share.created_at, zone_share.created_at)
        self.assertEqual(orig_zone_share.updated_at, zone_share.updated_at)

    def test_find_zone_share(self):
        orig_zone_share = self.operator_cloud.dns.create_zone_share(
            self.zone, target_project_id=self.demo_project_id
        )
        self.addCleanup(
            self.operator_cloud.dns.delete_zone_share,
            self.zone,
            orig_zone_share,
        )

        zone_share = self.operator_cloud.dns.find_zone_share(
            self.zone, orig_zone_share.id, ignore_missing=False
        )

        self.assertEqual(self.zone.id, zone_share.zone_id)
        self.assertEqual(self.project_id, zone_share.project_id)
        self.assertEqual(self.demo_project_id, zone_share.target_project_id)
        self.assertEqual(orig_zone_share.id, zone_share.id)
        self.assertEqual(orig_zone_share.created_at, zone_share.created_at)
        self.assertEqual(orig_zone_share.updated_at, zone_share.updated_at)

    def test_find_zone_share_ignore_missing(self):
        zone_share = self.operator_cloud.dns.find_zone_share(
            self.zone,
            'bogus_id',
        )
        self.assertIsNone(zone_share)

    def test_find_zone_share_ignore_missing_false(self):
        self.assertRaises(
            exceptions.NotFoundException,
            self.operator_cloud.dns.find_zone_share,
            self.zone,
            'bogus_id',
            ignore_missing=False,
        )

    def test_list_zone_shares(self):
        zone_share = self.operator_cloud.dns.create_zone_share(
            self.zone,
            target_project_id=self.demo_project_id,
        )
        self.addCleanup(
            self.operator_cloud.dns.delete_zone_share,
            self.zone,
            zone_share,
        )

        target_ids = [
            o.target_project_id
            for o in self.operator_cloud.dns.zone_shares(self.zone)
        ]
        self.assertIn(self.demo_project_id, target_ids)

    def test_list_zone_shares_with_target_id(self):
        zone_share = self.operator_cloud.dns.create_zone_share(
            self.zone,
            target_project_id=self.demo_project_id,
        )
        self.addCleanup(
            self.operator_cloud.dns.delete_zone_share,
            self.zone,
            zone_share,
        )

        target_ids = [
            o.target_project_id
            for o in self.operator_cloud.dns.zone_shares(
                self.zone, target_project_id=self.demo_project_id
            )
        ]
        self.assertIn(self.demo_project_id, target_ids)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any, ClassVar, Literal, overload
from collections.abc import Callable, Generator

from openstack.identity.v2 import extension as _extension
from openstack.identity.v2 import role as _role
from openstack.identity.v2 import tenant as _tenant
from openstack.identity.v2 import user as _user
from openstack import proxy
from openstack import resource


class Proxy(proxy.Proxy):
    api_version: ClassVar[Literal['2']] = '2'

    def extensions(self) -> Generator[_extension.Extension, None, None]:
        """Retrieve a generator of extensions

        :returns: A generator of extension instances.
        """
        return self._list(_extension.Extension)

    def get_extension(
        self, extension: str | _extension.Extension
    ) -> _extension.Extension:
        """Get a single extension

        :param extension: The value can be the ID of an extension or a
            :class:`~openstack.identity.v2.extension.Extension`
            instance.

        :returns: One :class:`~openstack.identity.v2.extension.Extension`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no extension can be found.
        """
        return self._get(_extension.Extension, extension)

    def create_role(self, **attrs: Any) -> _role.Role:
        """Create a new role from attributes

        :param attrs: Keyword arguments which will be used to create
            a :class:`~openstack.identity.v2.role.Role`,
            comprised of the properties on the Role class.

        :returns: The results of role creation
        """
        return self._create(_role.Role, **attrs)

    def delete_role(
        self, role: str | _role.Role, ignore_missing: bool = True
    ) -> None:
        """Delete a role

        :param role: The value can be either the ID of a role or a
            :class:`~openstack.identity.v2.role.Role` instance.
        :param ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the role does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent role.

        :returns: ``None``
        """
        self._delete(_role.Role, role, ignore_missing=ignore_missing)

    @overload
    def find_role(
        self,
        name_or_id: str,
        ignore_missing: Literal[False],
    ) -> _role.Role: ...

    @overload
    def find_role(
        self,
        name_or_id: str,
        ignore_missing: bool = True,
    ) -> _role.Role | None: ...

    def find_role(
        self,
        name_or_id: str,
        ignore_missing: bool = True,
    ) -> _role.Role | None:
        """Find a single role

        :param name_or_id: The name or ID of a role.
        :param ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.identity.v2.role.Role` or None
        """
        return self._find(
            _role.Role, name_or_id, ignore_missing=ignore_missing
        )

    def get_role(self, role: str | _role.Role) -> _role.Role:
        """Get a single role

        :param role: The value can be the ID of a role or a
            :class:`~openstack.identity.v2.role.Role` instance.

        :returns: One :class:`~openstack.identity.v2.role.Role`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_role.Role, role)

    def roles(self, **query: Any) -> Generator[_role.Role, None, None]:
        """Retrieve a generator of roles

        :param query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of role instances.
        """
        return self._list(_role.Role, **query)

    def update_role(self, role: str | _role.Role, **attrs: Any) -> _role.Role:
        """Update a role

        :param role: Either the ID of a role or a
            :class:`~openstack.identity.v2.role.Role` instance.
        :param attrs: The attributes to update on the role represented
            by ``role``.

        :returns: The updated role
        """
        return self._update(_role.Role, role, **attrs)

    def create_tenant(self, **attrs: Any) -> _tenant.Tenant:
        """Create a new tenant from attributes

        :param attrs: Keyword arguments which will be used to create
            a :class:`~openstack.identity.v2.tenant.Tenant`,
            comprised of the properties on the Tenant class.

        :returns: The results of tenant creation
        """
        return self._create(_tenant.Tenant, **attrs)

    def delete_tenant(
        self, tenant: str | _tenant.Tenant, ignore_missing: bool = True
    ) -> None:
        """Delete a tenant

        :param tenant: The value can be either the ID of a tenant or a
            :class:`~openstack.identity.v2.tenant.Tenant` instance.
        :param ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the tenant does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent tenant.

        :returns: ``None``
        """
        self._delete(_tenant.Tenant, tenant, ignore_missing=ignore_missing)

    @overload
    def find_tenant(
        self,
        name_or_id: str,
        ignore_missing: Literal[False],
    ) -> _tenant.Tenant: ...

    @overload
    def find_tenant(
        self,
        name_or_id: str,
        ignore_missing: bool = True,
    ) -> _tenant.Tenant | None: ...

    def find_tenant(
        self,
        name_or_id: str,
        ignore_missing: bool = True,
    ) -> _tenant.Tenant | None:
        """Find a single tenant

        :param name_or_id: The name or ID of a tenant.
        :param ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.identity.v2.tenant.Tenant` or None
        """
        return self._find(
            _tenant.Tenant, name_or_id, ignore_missing=ignore_missing
        )

    def get_tenant(self, tenant: str | _tenant.Tenant) -> _tenant.Tenant:
        """Get a single tenant

        :param tenant: The value can be the ID of a tenant or a
            :class:`~openstack.identity.v2.tenant.Tenant` instance.

        :returns: One :class:`~openstack.identity.v2.tenant.Tenant`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_tenant.Tenant, tenant)

    def tenants(self, **query: Any) -> Generator[_tenant.Tenant, None, None]:
        """Retrieve a generator of tenants

        :param query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of tenant instances.
        """
        return self._list(_tenant.Tenant, **query)

    def update_tenant(
        self, tenant: str | _tenant.Tenant, **attrs: Any
    ) -> _tenant.Tenant:
        """Update a tenant

        :param tenant: Either the ID of a tenant or a
            :class:`~openstack.identity.v2.tenant.Tenant` instance.
        :param attrs: The attributes to update on the tenant represented
            by ``tenant``.

        :returns: The updated tenant
        """
        return self._update(_tenant.Tenant, tenant, **attrs)

    def create_user(self, **attrs: Any) -> _user.User:
        """Create a new user from attributes

        :param attrs: Keyword arguments which will be used to create
            a :class:`~openstack.identity.v2.user.User`,
            comprised of the properties on the User class.

        :returns: The results of user creation
        """
        return self._create(_user.User, **attrs)

    def delete_user(
        self, user: str | _user.User, ignore_missing: bool = True
    ) -> None:
        """Delete a user

        :param user: The value can be either the ID of a user or a
            :class:`~openstack.identity.v2.user.User` instance.
        :param ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the user does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent user.

        :returns: ``None``
        """
        self._delete(_user.User, user, ignore_missing=ignore_missing)

    @overload
    def find_user(
        self,
        name_or_id: str,
        ignore_missing: Literal[False],
    ) -> _user.User: ...

    @overload
    def find_user(
        self,
        name_or_id: str,
        ignore_missing: bool = True,
    ) -> _user.User | None: ...

    def find_user(
        self,
        name_or_id: str,
        ignore_missing: bool = True,
    ) -> _user.User | None:
        """Find a single user

        :param name_or_id: The name or ID of a user.
        :param ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.identity.v2.user.User` or None
        """
        return self._find(
            _user.User, name_or_id, ignore_missing=ignore_missing
        )

    def get_user(self, user: str | _user.User) -> _user.User:
        """Get a single user

        :param user: The value can be the ID of a user or a
            :class:`~openstack.identity.v2.user.User` instance.

        :returns: One :class:`~openstack.identity.v2.user.User`
        :raises: :class:`~openstack.exceptions.NotFoundException`
            when no resource can be found.
        """
        return self._get(_user.User, user)

    def users(self, **query: Any) -> Generator[_user.User, None, None]:
        """Retrieve a generator of users

        :param query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of user instances.
        """
        return self._list(_user.User, **query)

    def update_user(self, user: str | _user.User, **attrs: Any) -> _user.User:
        """Update a user

        :param user: Either the ID of a user or a
            :class:`~openstack.identity.v2.user.User` instance.
        :param attrs: The attributes to update on the user represented
            by ``user``.

        :returns: The updated user
        """
        return self._update(_user.User, user, **attrs)

    # ========== Utilities ==========

    def wait_for_status(
        self,
        res: resource.ResourceT,
        status: str,
        failures: list[str] | None = None,
        interval: int | float | None = 2,
        wait: int | None = None,
        attribute: str = 'status',
        callback: Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for the resource to be in a particular status.

        :param session: The session to use for making this request.
        :param resource: The resource to wait on to reach the status. The
            resource must have a status attribute specified via ``attribute``.
        :param status: Desired status of the resource.
        :param failures: Statuses that would indicate the transition
            failed such as 'ERROR'. Defaults to ['ERROR'].
        :param interval: Number of seconds to wait between checks.
        :param wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :param attribute: Name of the resource attribute that contains the
            status.
        :param callback: A callback function. This will be called with a single
            value, progress. This is API specific but is generally a percentage
            value from 0-100.

        :returns: The updated resource.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
            transition to status failed to occur in ``wait`` seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
            transitioned to one of the states in ``failures``.
        :raises: :class:`~AttributeError` if the resource does not have a
            ``status`` attribute
        """
        return resource.wait_for_status(
            self, res, status, failures, interval, wait, attribute, callback
        )

    def wait_for_delete(
        self,
        res: resource.ResourceT,
        interval: int | float | None = 2,
        wait: int | None = 120,
        callback: Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for a resource to be deleted.

        :param res: The resource to wait on to be deleted.
        :param interval: Number of seconds to wait before to consecutive
            checks.
        :param wait: Maximum number of seconds to wait before the change.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self, res, interval, wait, callback)



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_devstack
-------------

Throw errors if we do not actually detect the services we're supposed to.
"""

import os

from testscenarios import load_tests_apply_scenarios as load_tests  # noqa

from openstack.tests.functional import base


class TestDevstack(base.BaseFunctionalTest):
    scenarios = [
        ('designate', dict(env='DESIGNATE', service='dns')),
        ('heat', dict(env='HEAT', service='orchestration')),
        (
            'magnum',
            dict(env='MAGNUM', service='container-infrastructure-management'),
        ),
        ('neutron', dict(env='NEUTRON', service='network')),
        ('octavia', dict(env='OCTAVIA', service='load-balancer')),
        ('swift', dict(env='SWIFT', service='object-store')),
    ]

    env: str
    service: str

    def test_has_service(self):
        if os.environ.get(f'OPENSTACKSDK_HAS_{self.env}', '0') == '1':
            self.assertTrue(self.user_cloud.has_service(self.service))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.shared_file_system.v2 import quota_set
from openstack.tests.unit import base

EXAMPLE = {
    "gigabytes": 1000,
    "shares": 50,
    "snapshot_gigabytes": 1000,
    "snapshots": 50,
    "id": "16e1ab15c35a457e9c2b2aa189f544e1",
    "share_networks": 10,
    "share_groups": 10,
    "share_group_snapshots": 10,
    "share_replicas": 100,
    "replica_gigabytes": 1000,
    "per_share_gigabytes": -1,
    "backups": 50,
    "backup_gigabytes": 1000,
}


class TestQuotaSet(base.TestCase):
    def test_basic(self):
        sot = quota_set.QuotaSet()
        self.assertEqual("quota_sets", sot.resources_key)
        self.assertEqual("/quota-sets/%(project_id)s", sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.requires_id)

    def test_create_quota_set(self):
        sot = quota_set.QuotaSet(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['gigabytes'], sot.gigabytes)
        self.assertEqual(EXAMPLE['shares'], sot.shares)
        self.assertEqual(EXAMPLE['snapshot_gigabytes'], sot.snapshot_gigabytes)
        self.assertEqual(EXAMPLE['share_networks'], sot.share_networks)
        self.assertEqual(EXAMPLE['share_replicas'], sot.share_replicas)
        self.assertEqual(
            EXAMPLE['per_share_gigabytes'], sot.per_share_gigabytes
        )
        self.assertEqual(EXAMPLE['backups'], sot.backups)



# Generated file, to change, run tools/print-services.py
from typing import TYPE_CHECKING

from openstack import service_description
from openstack.accelerator import accelerator_service
from openstack.baremetal import baremetal_service
from openstack.baremetal_introspection import baremetal_introspection_service
from openstack.block_storage import block_storage_service
from openstack.clustering import clustering_service
from openstack.compute import compute_service
from openstack.container_infrastructure_management import (
    container_infrastructure_management_service,
)
from openstack.database import database_service
from openstack.dns import dns_service
from openstack.identity import identity_service
from openstack.image import image_service
from openstack.instance_ha import instance_ha_service
from openstack.key_manager import key_manager_service
from openstack.load_balancer import load_balancer_service
from openstack.message import message_service
from openstack.network import network_service
from openstack.object_store import object_store_service
from openstack.orchestration import orchestration_service
from openstack.placement import placement_service
from openstack.shared_file_system import shared_file_system_service
from openstack.workflow import workflow_service

if TYPE_CHECKING:
    # the noqa is necessary as 'proxy' is only referenced in string subscripts
    # and ruff doesn't scan for name usage since they're not in annotation
    # positions
    from openstack import proxy  # noqa: F401


class ServicesMixin:
    identity = identity_service.IdentityService(service_type='identity')

    compute = compute_service.ComputeService(service_type='compute')

    image = image_service.ImageService(service_type='image')

    load_balancer = load_balancer_service.LoadBalancerService(
        service_type='load-balancer'
    )

    object_store = object_store_service.ObjectStoreService(
        service_type='object-store'
    )

    clustering = clustering_service.ClusteringService(
        service_type='clustering'
    )
    resource_cluster = clustering
    cluster = clustering

    data_processing = service_description.ServiceDescription['proxy.Proxy'](
        service_type='data-processing'
    )

    baremetal = baremetal_service.BaremetalService(service_type='baremetal')
    bare_metal = baremetal

    baremetal_introspection = (
        baremetal_introspection_service.BaremetalIntrospectionService(
            service_type='baremetal-introspection'
        )
    )

    key_manager = key_manager_service.KeyManagerService(
        service_type='key-manager'
    )

    resource_optimization = service_description.ServiceDescription[
        'proxy.Proxy'
    ](service_type='resource-optimization')
    infra_optim = resource_optimization

    message = message_service.MessageService(service_type='message')
    messaging = message

    application_catalog = service_description.ServiceDescription[
        'proxy.Proxy'
    ](service_type='application-catalog')

    container_infrastructure_management = container_infrastructure_management_service.ContainerInfrastructureManagementService(
        service_type='container-infrastructure-management'
    )
    container_infra = container_infrastructure_management
    container_infrastructure = container_infrastructure_management

    search = service_description.ServiceDescription['proxy.Proxy'](
        service_type='search'
    )

    dns = dns_service.DnsService(service_type='dns')

    workflow = workflow_service.WorkflowService(service_type='workflow')

    rating = service_description.ServiceDescription['proxy.Proxy'](
        service_type='rating'
    )

    operator_policy = service_description.ServiceDescription['proxy.Proxy'](
        service_type='operator-policy'
    )
    policy = operator_policy

    shared_file_system = shared_file_system_service.SharedFilesystemService(
        service_type='shared-file-system'
    )
    share = shared_file_system

    data_protection_orchestration = service_description.ServiceDescription[
        'proxy.Proxy'
    ](service_type='data-protection-orchestration')

    orchestration = orchestration_service.OrchestrationService(
        service_type='orchestration'
    )

    block_storage = block_storage_service.BlockStorageService(
        service_type='block-storage'
    )
    block_store = block_storage
    volume = block_storage

    alarm = service_description.ServiceDescription['proxy.Proxy'](
        service_type='alarm'
    )
    alarming = alarm

    meter = service_description.ServiceDescription['proxy.Proxy'](
        service_type='meter'
    )
    metering = meter
    telemetry = meter

    event = service_description.ServiceDescription['proxy.Proxy'](
        service_type='event'
    )
    events = event

    application_deployment = service_description.ServiceDescription[
        'proxy.Proxy'
    ](service_type='application-deployment')
    application_deployment = application_deployment

    multi_region_network_automation = service_description.ServiceDescription[
        'proxy.Proxy'
    ](service_type='multi-region-network-automation')
    tricircle = multi_region_network_automation

    database = database_service.DatabaseService(service_type='database')

    application_container = service_description.ServiceDescription[
        'proxy.Proxy'
    ](service_type='application-container')
    container = application_container

    root_cause_analysis = service_description.ServiceDescription[
        'proxy.Proxy'
    ](service_type='root-cause-analysis')
    rca = root_cause_analysis

    nfv_orchestration = service_description.ServiceDescription['proxy.Proxy'](
        service_type='nfv-orchestration'
    )

    network = network_service.NetworkService(service_type='network')

    backup = service_description.ServiceDescription['proxy.Proxy'](
        service_type='backup'
    )

    monitoring_logging = service_description.ServiceDescription['proxy.Proxy'](
        service_type='monitoring-logging'
    )
    monitoring_log_api = monitoring_logging

    monitoring = service_description.ServiceDescription['proxy.Proxy'](
        service_type='monitoring'
    )

    monitoring_events = service_description.ServiceDescription['proxy.Proxy'](
        service_type='monitoring-events'
    )

    placement = placement_service.PlacementService(service_type='placement')

    instance_ha = instance_ha_service.InstanceHaService(
        service_type='instance-ha'
    )
    ha = instance_ha

    reservation = service_description.ServiceDescription['proxy.Proxy'](
        service_type='reservation'
    )

    function_engine = service_description.ServiceDescription['proxy.Proxy'](
        service_type='function-engine'
    )

    accelerator = accelerator_service.AcceleratorService(
        service_type='accelerator'
    )

    admin_logic = service_description.ServiceDescription['proxy.Proxy'](
        service_type='admin-logic'
    )
    registration = admin_logic



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import message
from openstack.tests.unit import base

MESSAGE = {
    "request_id": "req-c1216709-afba-4703-a1a3-22eda88f2f5a",
    "links": [
        {"href": "http://localhost:8776/link1", "rel": "self"},
        {"href": "http://localhost:8776/link2", "rel": "bookmark"},
    ],
    "message_level": "ERROR",
    "event_id": "VOLUME_000002",
    "created_at": "2021-02-25T00:00:00-00:00",
    "guaranteed_until": "2021-02-26T00:00:00-00:00",
    "resource_uuid": "d5f6c517-c3e8-45fe-b994-b11118e4cacf",
    "id": "c506cd4b-9048-43bc-97ef-0d7dec369b42",
    "resource_type": "VOLUME",
    "user_message": "Test message.",
}


class TestMessage(base.TestCase):
    def test_basic(self):
        message_resource = message.Message()
        self.assertEqual('messages', message_resource.resource_key)
        self.assertEqual('messages', message_resource.resources_key)
        self.assertEqual('/messages', message_resource.base_path)
        self.assertTrue(message_resource.allow_fetch)
        self.assertFalse(message_resource.allow_create)
        self.assertFalse(message_resource.allow_commit)
        self.assertTrue(message_resource.allow_delete)
        self.assertTrue(message_resource.allow_list)

    def test_make_message(self):
        message_resource = message.Message(**MESSAGE)
        self.assertEqual(MESSAGE['request_id'], message_resource.request_id)
        self.assertEqual(
            MESSAGE['message_level'], message_resource.message_level
        )
        self.assertEqual(MESSAGE['event_id'], message_resource.event_id)
        self.assertEqual(MESSAGE['created_at'], message_resource.created_at)
        self.assertEqual(
            MESSAGE['guaranteed_until'], message_resource.guaranteed_until
        )
        self.assertEqual(
            MESSAGE['resource_uuid'], message_resource.resource_uuid
        )
        self.assertEqual(MESSAGE['id'], message_resource.id)
        self.assertEqual(
            MESSAGE['resource_type'], message_resource.resource_type
        )
        self.assertEqual(
            MESSAGE['user_message'], message_resource.user_message
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from examples.network import create as network_create
from examples.network import delete as network_delete
from examples.network import find as network_find
from examples.network import list as network_list

from openstack.tests.functional import base


class TestNetwork(base.BaseFunctionalTest):
    """Test the network examples

    The purpose of these tests is to ensure the examples run successfully.
    """

    def test_network(self):
        network_list.list_networks(self.operator_cloud)
        network_list.list_subnets(self.operator_cloud)
        network_list.list_ports(self.operator_cloud)
        network_list.list_security_groups(self.operator_cloud)
        network_list.list_routers(self.operator_cloud)

        network_find.find_network(self.operator_cloud)

        network_create.create_network(self.operator_cloud)
        network_delete.delete_network(self.operator_cloud)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.identity.v3 import _proxy
from openstack.identity.v3 import access_rule
from openstack.identity.v3 import credential
from openstack.identity.v3 import domain
from openstack.identity.v3 import domain_config
from openstack.identity.v3 import endpoint
from openstack.identity.v3 import group
from openstack.identity.v3 import policy
from openstack.identity.v3 import project
from openstack.identity.v3 import region
from openstack.identity.v3 import role
from openstack.identity.v3 import role_domain_group_assignment
from openstack.identity.v3 import role_domain_user_assignment
from openstack.identity.v3 import role_project_group_assignment
from openstack.identity.v3 import role_project_user_assignment
from openstack.identity.v3 import role_system_group_assignment
from openstack.identity.v3 import role_system_user_assignment
from openstack.identity.v3 import service
from openstack.identity.v3 import service_provider
from openstack.identity.v3 import trust
from openstack.identity.v3 import user
from openstack.tests.unit import test_proxy_base

USER_ID = 'user-id-' + uuid.uuid4().hex
ENDPOINT_ID = 'endpoint-id-' + uuid.uuid4().hex
PROJECT_ID = 'project-id-' + uuid.uuid4().hex


class TestIdentityProxyBase(test_proxy_base.TestProxyBase):
    def setUp(self):
        super().setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestIdentityProxyCredential(TestIdentityProxyBase):
    def test_credential_create_attrs(self):
        self.verify_create(self.proxy.create_credential, credential.Credential)

    def test_credential_delete(self):
        self.verify_delete(
            self.proxy.delete_credential, credential.Credential, False
        )

    def test_credential_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_credential, credential.Credential, True
        )

    def test_credential_find(self):
        self.verify_find(self.proxy.find_credential, credential.Credential)

    def test_credential_get(self):
        self.verify_get(self.proxy.get_credential, credential.Credential)

    def test_credentials(self):
        self.verify_list(self.proxy.credentials, credential.Credential)

    def test_credential_update(self):
        self.verify_update(self.proxy.update_credential, credential.Credential)


class TestIdentityProxyDomain(TestIdentityProxyBase):
    def test_domain_create_attrs(self):
        self.verify_create(self.proxy.create_domain, domain.Domain)

    def test_domain_delete(self):
        self.verify_delete(self.proxy.delete_domain, domain.Domain, False)

    def test_domain_delete_ignore(self):
        self.verify_delete(self.proxy.delete_domain, domain.Domain, True)

    def test_domain_find(self):
        self.verify_find(self.proxy.find_domain, domain.Domain)

    def test_domain_get(self):
        self.verify_get(self.proxy.get_domain, domain.Domain)

    def test_domains(self):
        self.verify_list(self.proxy.domains, domain.Domain)

    def test_domain_update(self):
        self.verify_update(self.proxy.update_domain, domain.Domain)


class TestIdentityProxyDomainConfig(TestIdentityProxyBase):
    def test_domain_config_create_attrs(self):
        self.verify_create(
            self.proxy.create_domain_config,
            domain_config.DomainConfig,
            method_args=['domain_id'],
            method_kwargs={},
            expected_args=[],
            expected_kwargs={
                'domain_id': 'domain_id',
            },
        )

    def test_domain_config_delete(self):
        self.verify_delete(
            self.proxy.delete_domain_config,
            domain_config.DomainConfig,
            ignore_missing=False,
            method_args=['domain_id'],
            method_kwargs={},
            expected_args=[None],
            expected_kwargs={
                'domain_id': 'domain_id',
            },
        )

    def test_domain_config_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_domain_config,
            domain_config.DomainConfig,
            ignore_missing=True,
            method_args=['domain_id'],
            method_kwargs={},
            expected_args=[None],
            expected_kwargs={
                'domain_id': 'domain_id',
            },
        )

    # no find_domain_config

    def test_domain_config_get(self):
        self.verify_get(
            self.proxy.get_domain_config,
            domain_config.DomainConfig,
            method_args=['domain_id'],
            method_kwargs={},
            expected_args=[],
            expected_kwargs={
                'domain_id': 'domain_id',
                'requires_id': False,
            },
        )

    # no domain_configs

    def test_domain_config_update(self):
        self.verify_update(
            self.proxy.update_domain_config,
            domain_config.DomainConfig,
            method_args=['domain_id'],
            method_kwargs={},
            expected_args=[None],
            expected_kwargs={
                'domain_id': 'domain_id',
            },
        )


class TestIdentityProxyEndpoint(TestIdentityProxyBase):
    def test_endpoint_create_attrs(self):
        self.verify_create(self.proxy.create_endpoint, endpoint.Endpoint)

    def test_endpoint_delete(self):
        self.verify_delete(
            self.proxy.delete_endpoint, endpoint.Endpoint, False
        )

    def test_endpoint_delete_ignore(self):
        self.verify_delete(self.proxy.delete_endpoint, endpoint.Endpoint, True)

    def test_endpoint_find(self):
        self.verify_find(self.proxy.find_endpoint, endpoint.Endpoint)

    def test_endpoint_get(self):
        self.verify_get(self.proxy.get_endpoint, endpoint.Endpoint)

    def test_endpoints(self):
        self.verify_list(self.proxy.endpoints, endpoint.Endpoint)

    def test_project_endpoints(self):
        self.verify_list(
            self.proxy.project_endpoints,
            endpoint.ProjectEndpoint,
            method_kwargs={'project': PROJECT_ID},
            expected_kwargs={'project_id': PROJECT_ID},
        )

    def test_endpoint_update(self):
        self.verify_update(self.proxy.update_endpoint, endpoint.Endpoint)


class TestIdentityProxyGroup(TestIdentityProxyBase):
    def test_group_create_attrs(self):
        self.verify_create(self.proxy.create_group, group.Group)

    def test_group_delete(self):
        self.verify_delete(self.proxy.delete_group, group.Group, False)

    def test_group_delete_ignore(self):
        self.verify_delete(self.proxy.delete_group, group.Group, True)

    def test_group_find(self):
        self.verify_find(self.proxy.find_group, group.Group)

    def test_group_get(self):
        self.verify_get(self.proxy.get_group, group.Group)

    def test_groups(self):
        self.verify_list(self.proxy.groups, group.Group)

    def test_group_update(self):
        self.verify_update(self.proxy.update_group, group.Group)

    def test_add_user_to_group(self):
        self._verify(
            "openstack.identity.v3.group.Group.add_user",
            self.proxy.add_user_to_group,
            method_args=['uid', 'gid'],
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
            ],
        )

    def test_remove_user_from_group(self):
        self._verify(
            "openstack.identity.v3.group.Group.remove_user",
            self.proxy.remove_user_from_group,
            method_args=['uid', 'gid'],
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
            ],
        )

    def test_check_user_in_group(self):
        self._verify(
            "openstack.identity.v3.group.Group.check_user",
            self.proxy.check_user_in_group,
            method_args=['uid', 'gid'],
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
            ],
        )

    def test_group_users(self):
        self.verify_list(
            self.proxy.group_users,
            user.User,
            method_kwargs={"group": 'group', "attrs": 1},
            expected_kwargs={"attrs": 1},
        )


class TestIdentityProxyPolicy(TestIdentityProxyBase):
    def test_policy_create_attrs(self):
        self.verify_create(self.proxy.create_policy, policy.Policy)

    def test_policy_delete(self):
        self.verify_delete(self.proxy.delete_policy, policy.Policy, False)

    def test_policy_delete_ignore(self):
        self.verify_delete(self.proxy.delete_policy, policy.Policy, True)

    def test_policy_find(self):
        self.verify_find(self.proxy.find_policy, policy.Policy)

    def test_policy_get(self):
        self.verify_get(self.proxy.get_policy, policy.Policy)

    def test_policies(self):
        self.verify_list(self.proxy.policies, policy.Policy)

    def test_policy_update(self):
        self.verify_update(self.proxy.update_policy, policy.Policy)


class TestIdentityProxyProject(TestIdentityProxyBase):
    def test_project_create_attrs(self):
        self.verify_create(self.proxy.create_project, project.Project)

    def test_project_delete(self):
        self.verify_delete(self.proxy.delete_project, project.Project, False)

    def test_project_delete_ignore(self):
        self.verify_delete(self.proxy.delete_project, project.Project, True)

    def test_project_find(self):
        self.verify_find(self.proxy.find_project, project.Project)

    def test_project_get(self):
        self.verify_get(self.proxy.get_project, project.Project)

    def test_projects(self):
        self.verify_list(self.proxy.projects, project.Project)

    def test_user_projects(self):
        self.verify_list(
            self.proxy.user_projects,
            project.UserProject,
            method_kwargs={'user': USER_ID},
            expected_kwargs={'user_id': USER_ID},
        )

    def test_endpoint_projects(self):
        self.verify_list(
            self.proxy.endpoint_projects,
            project.EndpointProject,
            method_kwargs={'endpoint': ENDPOINT_ID},
            expected_kwargs={'endpoint_id': ENDPOINT_ID},
        )

    def test_project_update(self):
        self.verify_update(self.proxy.update_project, project.Project)

    def test_project_associate_endpoint(self):
        self._verify(
            'openstack.identity.v3.project.Project.associate_endpoint',
            self.proxy.associate_endpoint_with_project,
            method_args=['project_id', 'endpoint_id'],
            expected_args=[self.proxy, 'endpoint_id'],
        )

    def test_project_disassociate_endpoint(self):
        self._verify(
            'openstack.identity.v3.project.Project.disassociate_endpoint',
            self.proxy.disassociate_endpoint_from_project,
            method_args=['project_id', 'endpoint_id'],
            expected_args=[self.proxy, 'endpoint_id'],
        )


class TestIdentityProxyService(TestIdentityProxyBase):
    def test_service_create_attrs(self):
        self.verify_create(self.proxy.create_service, service.Service)

    def test_service_delete(self):
        self.verify_delete(self.proxy.delete_service, service.Service, False)

    def test_service_delete_ignore(self):
        self.verify_delete(self.proxy.delete_service, service.Service, True)

    def test_service_find(self):
        self.verify_find(self.proxy.find_service, service.Service)

    def test_service_get(self):
        self.verify_get(self.proxy.get_service, service.Service)

    def test_services(self):
        self.verify_list(self.proxy.services, service.Service)

    def test_service_update(self):
        self.verify_update(self.proxy.update_service, service.Service)


class TestIdentityProxyUser(TestIdentityProxyBase):
    def test_user_create_attrs(self):
        self.verify_create(self.proxy.create_user, user.User)

    def test_user_delete(self):
        self.verify_delete(self.proxy.delete_user, user.User, False)

    def test_user_delete_ignore(self):
        self.verify_delete(self.proxy.delete_user, user.User, True)

    def test_user_find(self):
        self.verify_find(self.proxy.find_user, user.User)

    def test_user_get(self):
        self.verify_get(self.proxy.get_user, user.User)

    def test_users(self):
        self.verify_list(self.proxy.users, user.User)

    def test_user_update(self):
        self.verify_update(self.proxy.update_user, user.User)

    def test_user_update_password(self):
        self._verify(
            "openstack.identity.v3.user.User.update_password",
            self.proxy.update_password,
            method_args=[USER_ID, "orig", "new"],
            expected_args=[self.proxy, "orig", "new"],
        )

    def test_user_groups(self):
        self.verify_list(
            self.proxy.user_groups,
            group.UserGroup,
            method_kwargs={"user": 'user'},
            expected_kwargs={"user_id": "user"},
        )


class TestIdentityProxyToken(TestIdentityProxyBase):
    def test_token_validate(self):
        self._verify(
            "openstack.identity.v3.token.Token.validate",
            self.proxy.validate_token,
            method_args=['token'],
            method_kwargs={'nocatalog': False, 'allow_expired': False},
            expected_args=[self.proxy, 'token'],
            expected_kwargs={'nocatalog': False, 'allow_expired': False},
        )

    def test_token_check(self):
        self._verify(
            "openstack.identity.v3.token.Token.check",
            self.proxy.check_token,
            method_args=['token'],
            method_kwargs={'allow_expired': False},
            expected_args=[self.proxy, 'token'],
            expected_kwargs={'allow_expired': False},
        )

    def test_token_revoke(self):
        self._verify(
            "openstack.identity.v3.token.Token.revoke",
            self.proxy.revoke_token,
            method_args=['token'],
            method_kwargs={},
            expected_args=[self.proxy, 'token'],
            expected_kwargs={},
        )


class TestIdentityProxyTrust(TestIdentityProxyBase):
    def test_trust_create_attrs(self):
        self.verify_create(self.proxy.create_trust, trust.Trust)

    def test_trust_delete(self):
        self.verify_delete(self.proxy.delete_trust, trust.Trust, False)

    def test_trust_delete_ignore(self):
        self.verify_delete(self.proxy.delete_trust, trust.Trust, True)

    def test_trust_find(self):
        self.verify_find(self.proxy.find_trust, trust.Trust)

    def test_trust_get(self):
        self.verify_get(self.proxy.get_trust, trust.Trust)

    def test_trusts(self):
        self.verify_list(self.proxy.trusts, trust.Trust)


class TestIdentityProxyRegion(TestIdentityProxyBase):
    def test_region_create_attrs(self):
        self.verify_create(self.proxy.create_region, region.Region)

    def test_region_delete(self):
        self.verify_delete(self.proxy.delete_region, region.Region, False)

    def test_region_delete_ignore(self):
        self.verify_delete(self.proxy.delete_region, region.Region, True)

    def test_region_find(self):
        self.verify_find(self.proxy.find_region, region.Region)

    def test_region_get(self):
        self.verify_get(self.proxy.get_region, region.Region)

    def test_regions(self):
        self.verify_list(self.proxy.regions, region.Region)

    def test_region_update(self):
        self.verify_update(self.proxy.update_region, region.Region)


class TestIdentityProxyRole(TestIdentityProxyBase):
    def test_role_create_attrs(self):
        self.verify_create(self.proxy.create_role, role.Role)

    def test_role_delete(self):
        self.verify_delete(self.proxy.delete_role, role.Role, False)

    def test_role_delete_ignore(self):
        self.verify_delete(self.proxy.delete_role, role.Role, True)

    def test_role_find(self):
        self.verify_find(self.proxy.find_role, role.Role)

    def test_role_get(self):
        self.verify_get(self.proxy.get_role, role.Role)

    def test_roles(self):
        self.verify_list(self.proxy.roles, role.Role)

    def test_role_update(self):
        self.verify_update(self.proxy.update_role, role.Role)


class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
    def test_role_assignments_filter__domain_user(self):
        self.verify_list(
            self.proxy.role_assignments_filter,
            role_domain_user_assignment.RoleDomainUserAssignment,
            method_kwargs={'domain': 'domain', 'user': 'user'},
            expected_kwargs={
                'domain_id': 'domain',
                'user_id': 'user',
            },
        )

    def test_role_assignments_filter__domain_group(self):
        self.verify_list(
            self.proxy.role_assignments_filter,
            role_domain_group_assignment.RoleDomainGroupAssignment,
            method_kwargs={'domain': 'domain', 'group': 'group'},
            expected_kwargs={
                'domain_id': 'domain',
                'group_id': 'group',
            },
        )

    def test_role_assignments_filter__project_user(self):
        self.verify_list(
            self.proxy.role_assignments_filter,
            role_project_user_assignment.RoleProjectUserAssignment,
            method_kwargs={'project': 'project', 'user': 'user'},
            expected_kwargs={
                'project_id': 'project',
                'user_id': 'user',
            },
        )

    def test_role_assignments_filter__project_group(self):
        self.verify_list(
            self.proxy.role_assignments_filter,
            role_project_group_assignment.RoleProjectGroupAssignment,
            method_kwargs={'project': 'project', 'group': 'group'},
            expected_kwargs={
                'project_id': 'project',
                'group_id': 'group',
            },
        )

    def test_role_assignments_filter__system_user(self):
        self.verify_list(
            self.proxy.role_assignments_filter,
            role_system_user_assignment.RoleSystemUserAssignment,
            method_kwargs={'system': 'system', 'user': 'user'},
            expected_kwargs={
                'system_id': 'system',
                'user_id': 'user',
            },
        )

    def test_role_assignments_filter__system_group(self):
        self.verify_list(
            self.proxy.role_assignments_filter,
            role_system_group_assignment.RoleSystemGroupAssignment,
            method_kwargs={'system': 'system', 'group': 'group'},
            expected_kwargs={
                'system_id': 'system',
                'group_id': 'group',
            },
        )

    def test_assign_domain_role_to_user(self):
        self._verify(
            "openstack.identity.v3.domain.Domain.assign_role_to_user",
            self.proxy.assign_domain_role_to_user,
            method_args=['dom_id'],
            method_kwargs={'user': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_unassign_domain_role_from_user(self):
        self._verify(
            "openstack.identity.v3.domain.Domain.unassign_role_from_user",
            self.proxy.unassign_domain_role_from_user,
            method_args=['dom_id'],
            method_kwargs={'user': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_validate_user_has_domain_role(self):
        self._verify(
            "openstack.identity.v3.domain.Domain.validate_user_has_role",
            self.proxy.validate_user_has_domain_role,
            method_args=['dom_id'],
            method_kwargs={'user': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_assign_domain_role_to_group(self):
        self._verify(
            "openstack.identity.v3.domain.Domain.assign_role_to_group",
            self.proxy.assign_domain_role_to_group,
            method_args=['dom_id'],
            method_kwargs={'group': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_unassign_domain_role_from_group(self):
        self._verify(
            "openstack.identity.v3.domain.Domain.unassign_role_from_group",
            self.proxy.unassign_domain_role_from_group,
            method_args=['dom_id'],
            method_kwargs={'group': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_validate_group_has_domain_role(self):
        self._verify(
            "openstack.identity.v3.domain.Domain.validate_group_has_role",
            self.proxy.validate_group_has_domain_role,
            method_args=['dom_id'],
            method_kwargs={'group': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_assign_project_role_to_user(self):
        self._verify(
            "openstack.identity.v3.project.Project.assign_role_to_user",
            self.proxy.assign_project_role_to_user,
            method_args=['dom_id'],
            method_kwargs={'user': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_unassign_project_role_from_user(self):
        self._verify(
            "openstack.identity.v3.project.Project.unassign_role_from_user",
            self.proxy.unassign_project_role_from_user,
            method_args=['dom_id'],
            method_kwargs={'user': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_validate_user_has_project_role(self):
        self._verify(
            "openstack.identity.v3.project.Project.validate_user_has_role",
            self.proxy.validate_user_has_project_role,
            method_args=['dom_id'],
            method_kwargs={'user': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_assign_project_role_to_group(self):
        self._verify(
            "openstack.identity.v3.project.Project.assign_role_to_group",
            self.proxy.assign_project_role_to_group,
            method_args=['dom_id'],
            method_kwargs={'group': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_unassign_project_role_from_group(self):
        self._verify(
            "openstack.identity.v3.project.Project.unassign_role_from_group",
            self.proxy.unassign_project_role_from_group,
            method_args=['dom_id'],
            method_kwargs={'group': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_validate_group_has_project_role(self):
        self._verify(
            "openstack.identity.v3.project.Project.validate_group_has_role",
            self.proxy.validate_group_has_project_role,
            method_args=['dom_id'],
            method_kwargs={'group': 'uid', 'role': 'rid'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
                False,
            ],
        )

    def test_assign_system_role_to_user(self):
        self._verify(
            "openstack.identity.v3.system.System.assign_role_to_user",
            self.proxy.assign_system_role_to_user,
            method_kwargs={'user': 'uid', 'role': 'rid', 'system': 'all'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
            ],
        )

    def test_unassign_system_role_from_user(self):
        self._verify(
            "openstack.identity.v3.system.System.unassign_role_from_user",
            self.proxy.unassign_system_role_from_user,
            method_kwargs={'user': 'uid', 'role': 'rid', 'system': 'all'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
            ],
        )

    def test_validate_user_has_system_role(self):
        self._verify(
            "openstack.identity.v3.system.System.validate_user_has_role",
            self.proxy.validate_user_has_system_role,
            method_kwargs={'user': 'uid', 'role': 'rid', 'system': 'all'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(user.User, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
            ],
        )

    def test_assign_system_role_to_group(self):
        self._verify(
            "openstack.identity.v3.system.System.assign_role_to_group",
            self.proxy.assign_system_role_to_group,
            method_kwargs={'group': 'uid', 'role': 'rid', 'system': 'all'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
            ],
        )

    def test_unassign_system_role_from_group(self):
        self._verify(
            "openstack.identity.v3.system.System.unassign_role_from_group",
            self.proxy.unassign_system_role_from_group,
            method_kwargs={'group': 'uid', 'role': 'rid', 'system': 'all'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
            ],
        )

    def test_validate_group_has_system_role(self):
        self._verify(
            "openstack.identity.v3.system.System.validate_group_has_role",
            self.proxy.validate_group_has_system_role,
            method_kwargs={'group': 'uid', 'role': 'rid', 'system': 'all'},
            expected_args=[
                self.proxy,
                self.proxy._get_resource(group.Group, 'uid'),
                self.proxy._get_resource(role.Role, 'rid'),
            ],
        )


class TestAccessRule(TestIdentityProxyBase):
    def test_access_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_access_rule,
            access_rule.AccessRule,
            False,
            method_args=[],
            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
            expected_args=['access_rule'],
            expected_kwargs={'user_id': USER_ID},
        )

    def test_access_rule_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_access_rule,
            access_rule.AccessRule,
            True,
            method_args=[],
            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
            expected_args=['access_rule'],
            expected_kwargs={'user_id': USER_ID},
        )

    def test_access_rule_get(self):
        self.verify_get(
            self.proxy.get_access_rule,
            access_rule.AccessRule,
            method_args=[],
            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
            expected_args=['access_rule'],
            expected_kwargs={'user_id': USER_ID},
        )

    def test_access_rules(self):
        self.verify_list(
            self.proxy.access_rules,
            access_rule.AccessRule,
            method_kwargs={'user': USER_ID},
            expected_kwargs={'user_id': USER_ID},
        )


class TestServiceProvider(TestIdentityProxyBase):
    def test_service_provider_create(self):
        self.verify_create(
            self.proxy.create_service_provider,
            service_provider.ServiceProvider,
        )

    def test_service_provider_delete(self):
        self.verify_delete(
            self.proxy.delete_service_provider,
            service_provider.ServiceProvider,
            False,
        )

    def test_service_provider_delete_ignore(self):
        self.verify_delete(
            self.proxy.delete_service_provider,
            service_provider.ServiceProvider,
            True,
        )

    def test_service_provider_find(self):
        self.verify_find(
            self.proxy.find_service_provider, service_provider.ServiceProvider
        )

    def test_service_provider_get(self):
        self.verify_get(
            self.proxy.get_service_provider,
            service_provider.ServiceProvider,
        )

    def test_service_providers(self):
        self.verify_list(
            self.proxy.service_providers,
            service_provider.ServiceProvider,
        )

    def test_service_provider_update(self):
        self.verify_update(
            self.proxy.update_service_provider,
            service_provider.ServiceProvider,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from typing import Any

from keystoneauth1 import adapter

from openstack import exceptions
from openstack.network.v2 import bgp_speaker as _speaker
from openstack import resource
from openstack import utils


class Agent(resource.Resource):
    """Neutron agent extension."""

    resource_key = 'agent'
    resources_key = 'agents'
    base_path = '/agents'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # NOTE: We skip query for JSON fields and datetime fields
    _query_mapping = resource.QueryParameters(
        'agent_type',
        'availability_zone',
        'binary',
        'description',
        'host',
        'topic',
        is_admin_state_up='admin_state_up',
        is_alive='alive',
    )

    # Properties
    #: The type of network agent.
    agent_type = resource.Body('agent_type')
    #: Availability zone for the network agent.
    availability_zone = resource.Body('availability_zone')
    #: The name of the network agent's application binary.
    binary = resource.Body('binary')
    #: Network agent configuration data specific to the agent_type.
    configuration = resource.Body('configurations')
    #: Timestamp when the network agent was created.
    created_at = resource.Body('created_at')
    #: The network agent description.
    description = resource.Body('description')
    #: Timestamp when the network agent's heartbeat was last seen.
    last_heartbeat_at = resource.Body('heartbeat_timestamp')
    #: The host the agent is running on.
    host = resource.Body('host')
    #: The administrative state of the network agent, which is up
    #: ``True`` or down ``False``. *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Whether or not the network agent is alive.
    #: *Type: bool*
    is_alive = resource.Body('alive', type=bool)
    #: Whether or not the agent is succesffully synced towards placement.
    #: Agents supporting the guaranteed minimum bandwidth feature share their
    #: resource view with neutron-server and neutron-server share this view
    #: with placement, resources_synced represents the success of the latter.
    #: The value None means no resource view synchronization to Placement was
    #: attempted. true / false values signify the success of the last
    #: synchronization attempt.
    #: *Type: bool*
    resources_synced = resource.Body('resources_synced', type=bool)
    #: Timestamp when the network agent was last started.
    started_at = resource.Body('started_at')
    #: The messaging queue topic the network agent subscribes to.
    topic = resource.Body('topic')
    #: The HA state of the L3 agent. This is one of 'active', 'standby' or
    #: 'fault' for HA routers, or None for other types of routers.
    ha_state = resource.Body('ha_state')
    #: The HA chassis priority for this agent in the router's
    #: HA_Chassis_Group (OVN). Higher values indicate higher priority;
    #: the highest priority chassis is the active gateway. None for
    #: non-OVN backends.
    #: *Type: int*
    ha_chassis_priority = resource.Body('ha_chassis_priority', type=int)

    def add_agent_to_network(
        self, session: adapter.Adapter, network_id: str
    ) -> dict[str, Any]:
        body = {'network_id': network_id}
        url = utils.urljoin(self.base_path, self.id, 'dhcp-networks')
        resp = session.post(url, json=body)
        return resp.json()  # type: ignore[no-any-return]

    def remove_agent_from_network(
        self, session: adapter.Adapter, network_id: str
    ) -> None:
        body = {'network_id': network_id}
        url = utils.urljoin(
            self.base_path, self.id, 'dhcp-networks', network_id
        )
        session.delete(url, json=body)

    def add_router_to_agent(
        self,
        session: adapter.Adapter,
        router: str,
        *,
        ha_chassis_priority: int | None = None,
    ) -> dict[str, Any]:
        body: dict[str, Any] = {'router_id': router}
        if ha_chassis_priority is not None:
            body['ha_chassis_priority'] = ha_chassis_priority
        url = utils.urljoin(self.base_path, self.id, 'l3-routers')
        resp = session.post(url, json=body)
        return resp.json()  # type: ignore[no-any-return]

    def update_router_in_agent(
        self,
        session: adapter.Adapter,
        router: str,
        *,
        ha_chassis_priority: int | None = None,
    ) -> dict[str, Any]:
        body: dict[str, Any] = {}
        if ha_chassis_priority is not None:
            body['ha_chassis_priority'] = ha_chassis_priority
        url = utils.urljoin(self.base_path, self.id, 'l3-routers', router)
        resp = session.put(url, json=body)
        return resp.json()  # type: ignore[no-any-return]

    def remove_router_from_agent(
        self, session: adapter.Adapter, router: str
    ) -> None:
        body = {'router_id': router}
        url = utils.urljoin(self.base_path, self.id, 'l3-routers', router)
        session.delete(url, json=body)

    def get_bgp_speakers_hosted_by_dragent(
        self, session: adapter.Adapter
    ) -> list[_speaker.BgpSpeaker]:
        """List BGP speakers hosted by a Dynamic Routing Agent

        :param session: The session to communicate through.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :returns: A list of BgpSpeakers
        :rtype: :class:`~openstack.network.v2.bgp_speaker.BgpSpeaker`
        """
        url = utils.urljoin(self.base_path, self.id, 'bgp-drinstances')
        resp = session.get(url)
        exceptions.raise_from_response(resp)
        self._body.attributes.update(resp.json())
        speaker_ids = [sp['id'] for sp in resp.json()['bgp_speakers']]
        speakers = _speaker.BgpSpeaker.list(session=session)
        return [sp for sp in speakers if sp.id in speaker_ids]


class NetworkHostingDHCPAgent(Agent):
    resource_key = 'agent'
    resources_key = 'agents'
    resource_name = 'dhcp-agent'
    base_path = '/networks/%(network_id)s/dhcp-agents'

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    # NOTE: Doesn't support query yet.


class RouterL3Agent(Agent):
    resource_key = 'agent'
    resources_key = 'agents'
    base_path = '/routers/%(router_id)s/l3-agents'
    resource_name = 'l3-agent'

    # capabilities
    allow_create = False
    allow_retrieve = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    # NOTE: No query parameter is supported



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import limits
from openstack.tests.unit import base

ABSOLUTE_LIMIT = {
    "totalSnapshotsUsed": 1,
    "maxTotalBackups": 10,
    "maxTotalVolumeGigabytes": 1000,
    "maxTotalSnapshots": 10,
    "maxTotalBackupGigabytes": 1000,
    "totalBackupGigabytesUsed": 1,
    "maxTotalVolumes": 10,
    "totalVolumesUsed": 2,
    "totalBackupsUsed": 3,
    "totalGigabytesUsed": 2,
}

RATE_LIMIT = {
    "verb": "POST",
    "value": 80,
    "remaining": 80,
    "unit": "MINUTE",
    "next-available": "2021-02-23T22:08:00Z",
}

RATE_LIMITS = {"regex": ".*", "uri": "*", "limit": [RATE_LIMIT]}

LIMIT = {"rate": [RATE_LIMITS], "absolute": ABSOLUTE_LIMIT}


class TestAbsoluteLimit(base.TestCase):
    def test_basic(self):
        limit_resource = limits.AbsoluteLimit()
        self.assertIsNone(limit_resource.resource_key)
        self.assertIsNone(limit_resource.resources_key)
        self.assertEqual('', limit_resource.base_path)
        self.assertFalse(limit_resource.allow_create)
        self.assertFalse(limit_resource.allow_fetch)
        self.assertFalse(limit_resource.allow_delete)
        self.assertFalse(limit_resource.allow_commit)
        self.assertFalse(limit_resource.allow_list)

    def test_make_absolute_limit(self):
        limit_resource = limits.AbsoluteLimit(**ABSOLUTE_LIMIT)
        self.assertEqual(
            ABSOLUTE_LIMIT['totalSnapshotsUsed'],
            limit_resource.total_snapshots_used,
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['maxTotalBackups'], limit_resource.max_total_backups
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['maxTotalVolumeGigabytes'],
            limit_resource.max_total_volume_gigabytes,
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['maxTotalSnapshots'],
            limit_resource.max_total_snapshots,
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['maxTotalBackupGigabytes'],
            limit_resource.max_total_backup_gigabytes,
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['totalBackupGigabytesUsed'],
            limit_resource.total_backup_gigabytes_used,
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['maxTotalVolumes'], limit_resource.max_total_volumes
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['totalVolumesUsed'],
            limit_resource.total_volumes_used,
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['totalBackupsUsed'],
            limit_resource.total_backups_used,
        )
        self.assertEqual(
            ABSOLUTE_LIMIT['totalGigabytesUsed'],
            limit_resource.total_gigabytes_used,
        )


class TestRateLimit(base.TestCase):
    def test_basic(self):
        limit_resource = limits.RateLimit()
        self.assertIsNone(limit_resource.resource_key)
        self.assertIsNone(limit_resource.resources_key)
        self.assertEqual('', limit_resource.base_path)
        self.assertFalse(limit_resource.allow_create)
        self.assertFalse(limit_resource.allow_fetch)
        self.assertFalse(limit_resource.allow_delete)
        self.assertFalse(limit_resource.allow_commit)
        self.assertFalse(limit_resource.allow_list)

    def test_make_rate_limit(self):
        limit_resource = limits.RateLimit(**RATE_LIMIT)
        self.assertEqual(RATE_LIMIT['verb'], limit_resource.verb)
        self.assertEqual(RATE_LIMIT['value'], limit_resource.value)
        self.assertEqual(RATE_LIMIT['remaining'], limit_resource.remaining)
        self.assertEqual(RATE_LIMIT['unit'], limit_resource.unit)
        self.assertEqual(
            RATE_LIMIT['next-available'], limit_resource.next_available
        )


class TestRateLimits(base.TestCase):
    def test_basic(self):
        limit_resource = limits.RateLimits()
        self.assertIsNone(limit_resource.resource_key)
        self.assertIsNone(limit_resource.resources_key)
        self.assertEqual('', limit_resource.base_path)
        self.assertFalse(limit_resource.allow_create)
        self.assertFalse(limit_resource.allow_fetch)
        self.assertFalse(limit_resource.allow_delete)
        self.assertFalse(limit_resource.allow_commit)
        self.assertFalse(limit_resource.allow_list)

    def _test_rate_limit(self, expected, actual):
        self.assertEqual(expected[0]['verb'], actual[0].verb)
        self.assertEqual(expected[0]['value'], actual[0].value)
        self.assertEqual(expected[0]['remaining'], actual[0].remaining)
        self.assertEqual(expected[0]['unit'], actual[0].unit)
        self.assertEqual(
            expected[0]['next-available'], actual[0].next_available
        )

    def test_make_rate_limits(self):
        limit_resource = limits.RateLimits(**RATE_LIMITS)
        self.assertEqual(RATE_LIMITS['regex'], limit_resource.regex)
        self.assertEqual(RATE_LIMITS['uri'], limit_resource.uri)
        self._test_rate_limit(RATE_LIMITS['limit'], limit_resource.limits)


class TestLimit(base.TestCase):
    def test_basic(self):
        limit_resource = limits.Limits()
        self.assertEqual('limits', limit_resource.resource_key)
        self.assertEqual('/limits', limit_resource.base_path)
        self.assertTrue(limit_resource.allow_fetch)
        self.assertFalse(limit_resource.allow_create)
        self.assertFalse(limit_resource.allow_commit)
        self.assertFalse(limit_resource.allow_delete)
        self.assertFalse(limit_resource.allow_list)

    def _test_absolute_limit(self, expected, actual):
        self.assertEqual(
            expected['totalSnapshotsUsed'], actual.total_snapshots_used
        )
        self.assertEqual(expected['maxTotalBackups'], actual.max_total_backups)
        self.assertEqual(
            expected['maxTotalVolumeGigabytes'],
            actual.max_total_volume_gigabytes,
        )
        self.assertEqual(
            expected['maxTotalSnapshots'], actual.max_total_snapshots
        )
        self.assertEqual(
            expected['maxTotalBackupGigabytes'],
            actual.max_total_backup_gigabytes,
        )
        self.assertEqual(
            expected['totalBackupGigabytesUsed'],
            actual.total_backup_gigabytes_used,
        )
        self.assertEqual(expected['maxTotalVolumes'], actual.max_total_volumes)
        self.assertEqual(
            expected['totalVolumesUsed'], actual.total_volumes_used
        )
        self.assertEqual(
            expected['totalBackupsUsed'], actual.total_backups_used
        )
        self.assertEqual(
            expected['totalGigabytesUsed'], actual.total_gigabytes_used
        )

    def _test_rate_limit(self, expected, actual):
        self.assertEqual(expected[0]['verb'], actual[0].verb)
        self.assertEqual(expected[0]['value'], actual[0].value)
        self.assertEqual(expected[0]['remaining'], actual[0].remaining)
        self.assertEqual(expected[0]['unit'], actual[0].unit)
        self.assertEqual(
            expected[0]['next-available'], actual[0].next_available
        )

    def _test_rate_limits(self, expected, actual):
        self.assertEqual(expected[0]['regex'], actual[0].regex)
        self.assertEqual(expected[0]['uri'], actual[0].uri)
        self._test_rate_limit(expected[0]['limit'], actual[0].limits)

    def test_make_limit(self):
        limit_resource = limits.Limits(**LIMIT)
        self._test_rate_limits(LIMIT['rate'], limit_resource.rate)
        self._test_absolute_limit(LIMIT['absolute'], limit_resource.absolute)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy
from unittest import mock

from keystoneauth1 import adapter

from openstack.common import quota_set as _qs
from openstack.tests.unit import base


BASIC_EXAMPLE = {
    "backup_gigabytes": 1000,
    "backups": 10,
    "gigabytes___DEFAULT__": -1,
}

USAGE_EXAMPLE = {
    "backup_gigabytes": {"in_use": 0, "limit": 1000, "reserved": 0},
    "backups": {"in_use": 0, "limit": 10, "reserved": 0},
    "gigabytes___DEFAULT__": {"in_use": 0, "limit": -1, "reserved": 0},
}


class TestQuotaSet(base.TestCase):
    def setUp(self):
        super().setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = 1
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sess.retriable_status_codes = set()

    def test_basic(self):
        sot = _qs.QuotaSet()
        self.assertEqual('quota_set', sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/os-quota-sets/%(project_id)s', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertTrue(sot.allow_commit)

        self.assertDictEqual(
            {"usage": "usage", "limit": "limit", "marker": "marker"},
            sot._query_mapping._mapping,
        )

    def test_make_basic(self):
        sot = _qs.QuotaSet(**BASIC_EXAMPLE)

        self.assertEqual(BASIC_EXAMPLE['backups'], sot.backups)

    def test_get(self):
        sot = _qs.QuotaSet(project_id='proj')

        resp = mock.Mock()
        resp.body = {'quota_set': copy.deepcopy(BASIC_EXAMPLE)}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        resp.headers = {}
        self.sess.get = mock.Mock(return_value=resp)

        sot.fetch(self.sess)

        self.sess.get.assert_called_with(
            '/os-quota-sets/proj', microversion=1, params={}, skip_cache=False
        )

        self.assertEqual(BASIC_EXAMPLE['backups'], sot.backups)
        self.assertEqual({}, sot.reservation)
        self.assertEqual({}, sot.usage)

    def test_get_usage(self):
        sot = _qs.QuotaSet(project_id='proj')

        resp = mock.Mock()
        resp.body = {'quota_set': copy.deepcopy(USAGE_EXAMPLE)}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        resp.headers = {}
        self.sess.get = mock.Mock(return_value=resp)

        sot.fetch(self.sess, usage=True)

        self.sess.get.assert_called_with(
            '/os-quota-sets/proj',
            microversion=1,
            params={'usage': True},
            skip_cache=False,
        )

        self.assertEqual(USAGE_EXAMPLE['backups']['limit'], sot.backups)

    def test_update_quota(self):
        # Use QuotaSet as if it was returned by get(usage=True)
        sot = _qs.QuotaSet.existing(
            project_id='proj',
            reservation={'a': 'b'},
            usage={'c': 'd'},
            foo='bar',
        )

        resp = mock.Mock()
        resp.body = {'quota_set': copy.deepcopy(BASIC_EXAMPLE)}
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        resp.headers = {}
        self.sess.put = mock.Mock(return_value=resp)

        sot._update(reservation={'b': 'd'}, backups=15, something_else=20)

        sot.commit(self.sess)

        self.sess.put.assert_called_with(
            '/os-quota-sets/proj',
            microversion=1,
            headers={},
            json={'quota_set': {'backups': 15, 'something_else': 20}},
        )

    def test_delete_quota(self):
        # Use QuotaSet as if it was returned by get(usage=True)
        sot = _qs.QuotaSet.existing(
            project_id='proj',
            reservation={'a': 'b'},
            usage={'c': 'd'},
            foo='bar',
        )

        resp = mock.Mock()
        resp.body = None
        resp.json = mock.Mock(return_value=resp.body)
        resp.status_code = 200
        resp.headers = {}
        self.sess.delete = mock.Mock(return_value=resp)

        sot.delete(self.sess)

        self.sess.delete.assert_called_with(
            '/os-quota-sets/proj',
            microversion=1,
            headers={},
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import network
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'admin_state_up': True,
    'availability_zone_hints': ['1', '2'],
    'availability_zones': ['3'],
    'created_at': '2016-03-09T12:14:57.233772',
    'description': '4',
    'dns_domain': '5',
    'id': IDENTIFIER,
    'ipv4_address_scope': '6',
    'ipv6_address_scope': '7',
    'is_default': False,
    'mtu': 8,
    'name': '9',
    'port_security_enabled': True,
    'project_id': '10',
    'provider:network_type': '11',
    'provider:physical_network': '12',
    'provider:segmentation_id': '13',
    'qos_policy_id': '14',
    'revision_number': 15,
    'router:external': True,
    'segments': ['16'],
    'shared': True,
    'status': '17',
    'subnets': ['18', '19'],
    'updated_at': '2016-07-09T12:14:57.233772',
    'vlan_transparent': False,
    'vlan_qinq': False,
    'pvlan': False,
}


class TestNetwork(base.TestCase):
    def test_basic(self):
        sot = network.Network()
        self.assertEqual('network', sot.resource_key)
        self.assertEqual('networks', sot.resources_key)
        self.assertEqual('/networks', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = network.Network(**EXAMPLE)
        self.assertTrue(sot.is_admin_state_up)
        self.assertEqual(
            EXAMPLE['availability_zone_hints'], sot.availability_zone_hints
        )
        self.assertEqual(EXAMPLE['availability_zones'], sot.availability_zones)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['dns_domain'], sot.dns_domain)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(
            EXAMPLE['ipv4_address_scope'], sot.ipv4_address_scope_id
        )
        self.assertEqual(
            EXAMPLE['ipv6_address_scope'], sot.ipv6_address_scope_id
        )
        self.assertFalse(sot.is_default)
        self.assertEqual(EXAMPLE['mtu'], sot.mtu)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertTrue(sot.is_port_security_enabled)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(
            EXAMPLE['provider:network_type'], sot.provider_network_type
        )
        self.assertEqual(
            EXAMPLE['provider:physical_network'], sot.provider_physical_network
        )
        self.assertEqual(
            EXAMPLE['provider:segmentation_id'], sot.provider_segmentation_id
        )
        self.assertEqual(EXAMPLE['qos_policy_id'], sot.qos_policy_id)
        self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
        self.assertTrue(sot.is_router_external)
        self.assertEqual(EXAMPLE['segments'], sot.segments)
        self.assertTrue(sot.is_shared)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['subnets'], sot.subnet_ids)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['vlan_transparent'], sot.is_vlan_transparent)
        self.assertEqual(EXAMPLE['vlan_qinq'], sot.is_vlan_qinq)
        self.assertEqual(EXAMPLE['pvlan'], sot.pvlan)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
                'description': 'description',
                'id': 'id',
                'name': 'name',
                'project_id': 'project_id',
                'status': 'status',
                'ipv4_address_scope_id': 'ipv4_address_scope',
                'ipv6_address_scope_id': 'ipv6_address_scope',
                'is_admin_state_up': 'admin_state_up',
                'is_port_security_enabled': 'port_security_enabled',
                'is_router_external': 'router:external',
                'is_shared': 'shared',
                'provider_network_type': 'provider:network_type',
                'provider_physical_network': 'provider:physical_network',
                'provider_segmentation_id': 'provider:segmentation_id',
                'tags': 'tags',
                'any_tags': 'tags-any',
                'not_tags': 'not-tags',
                'not_any_tags': 'not-tags-any',
                'sort_dir': 'sort_dir',
                'sort_key': 'sort_key',
            },
            sot._query_mapping._mapping,
        )


class TestDHCPAgentHostingNetwork(base.TestCase):
    def test_basic(self):
        net = network.DHCPAgentHostingNetwork()
        self.assertEqual('network', net.resource_key)
        self.assertEqual('networks', net.resources_key)
        self.assertEqual('/agents/%(agent_id)s/dhcp-networks', net.base_path)
        self.assertEqual('dhcp-network', net.resource_name)
        self.assertFalse(net.allow_create)
        self.assertTrue(net.allow_fetch)
        self.assertFalse(net.allow_commit)
        self.assertFalse(net.allow_delete)
        self.assertTrue(net.allow_list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.shared_file_system.v2 import share_network as _share_network
from openstack.tests.functional.shared_file_system.v2 import base


class ShareNetworkTest(base.BaseSharedFileSystemTest):
    def setUp(self):
        super().setUp()

        self.NETWORK_NAME = self.getUniqueString()
        net = self.user_cloud.network.create_network(name=self.NETWORK_NAME)
        self.assertIsNotNone(net)
        self.assertIsNotNone(net.id)
        self.NETWORK_ID = net.id

        self.SUBNET_NAME = self.getUniqueString()
        subnet = self.user_cloud.network.create_subnet(
            name=self.SUBNET_NAME,
            network_id=self.NETWORK_ID,
            ip_version=4,
            cidr='10.0.0.0/24',
        )
        self.SUBNET_ID = subnet.id

        self.SHARE_NETWORK_NAME = self.getUniqueString()
        snt = self.user_cloud.shared_file_system.create_share_network(
            name=self.SHARE_NETWORK_NAME,
            neutron_net_id=self.NETWORK_ID,
            neutron_subnet_id=self.SUBNET_ID,
        )

        self.assertIsNotNone(snt)
        self.assertIsNotNone(snt.id)
        self.SHARE_NETWORK_ID = snt.id

    def tearDown(self):
        sot = self.user_cloud.shared_file_system.delete_share_network(
            self.SHARE_NETWORK_ID, ignore_missing=True
        )
        self.assertIsNone(sot)
        self.user_cloud.network.delete_network(self.NETWORK_ID)
        super().tearDown()

    def test_get(self):
        sot = self.user_cloud.shared_file_system.get_share_network(
            self.SHARE_NETWORK_ID
        )
        assert isinstance(sot, _share_network.ShareNetwork)
        self.assertEqual(self.SHARE_NETWORK_ID, sot.id)
        self.assertIsNotNone(sot.share_network_subnets)
        self.assertEqual(
            self.NETWORK_ID,
            sot.share_network_subnets[0]['neutron_net_id'],
        )
        self.assertEqual(
            self.SUBNET_ID,
            sot.share_network_subnets[0]['neutron_subnet_id'],
        )

    def test_list_share_network(self):
        share_nets = self.user_cloud.shared_file_system.share_networks(
            details=False
        )
        self.assertGreater(len(list(share_nets)), 0)
        for share_net in share_nets:
            for attribute in ('id', 'name', 'created_at', 'updated_at'):
                self.assertTrue(hasattr(share_net, attribute))

    def test_delete_share_network(self):
        sot = self.user_cloud.shared_file_system.delete_share_network(
            self.SHARE_NETWORK_ID
        )
        self.assertIsNone(sot)

    def test_update(self):
        unt = self.user_cloud.shared_file_system.update_share_network(
            self.SHARE_NETWORK_ID, description='updated share network'
        )
        get_unt = self.user_cloud.shared_file_system.get_share_network(unt.id)
        self.assertEqual('updated share network', get_unt.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.baremetal.v1 import _common
from openstack import resource


class PortGroup(_common.Resource):
    resources_key = 'portgroups'
    base_path = '/portgroups'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_patch = True
    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters(
        'node',
        'address',
        fields={'type': _common.fields_type},
    )

    # The mode and properties field introduced in 1.26.
    _max_microversion = '1.26'

    #: The physical hardware address of the portgroup, typically the hardware
    #: MAC address. Added in API microversion 1.23.
    address = resource.Body('address')
    #: Timestamp at which the portgroup was created.
    created_at = resource.Body('created_at')
    #: A set of one or more arbitrary metadata key and value pairs.
    extra = resource.Body('extra', type=dict)
    #: The name of the portgroup
    name = resource.Body('name')
    #: The UUID for the portgroup
    id = resource.Body('uuid', alternate_id=True)
    #: Internal metadaa set and stored by the portgroup.
    internal_info = resource.Body('internal_info')
    #: Whether ports that are members of this portgroup can be used as
    #: standalone ports. Added in API microversion 1.23.
    is_standalone_ports_supported = resource.Body(
        'standalone_ports_supported', type=bool
    )
    #: A list of relative links, including the self and bookmark links.
    links = resource.Body('links', type=list)
    #: Port bonding mode. Added in API microversion 1.26.
    mode = resource.Body('mode')
    #: UUID of the node this portgroup belongs to.
    node_id = resource.Body('node_uuid')
    #: A list of links to the collection of ports belonging to this portgroup.
    #: Added in API microversion 1.24.
    ports = resource.Body('ports')
    #: Port group properties. Added in API microversion 1.26.
    properties = resource.Body('properties', type=dict)
    #: Timestamp at which the portgroup was last updated.
    updated_at = resource.Body('updated_at')


PortGroupDetail = PortGroup
